@@ -1,11 +1,10 @@
 /* $Id: VBoxDispDDrawVHWA.cpp $ */
-
 /** @file
  * VBox XPDM Display driver, DirectDraw callbacks VHWA related
  */
 
 /*
- * Copyright (C) 2011 Oracle Corporation
+ * Copyright (C) 2011-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -20,8 +19,9 @@
 #include "VBoxDispDDraw.h"
 #include <iprt/asm.h>
 
-static DECLCALLBACK(void) VBoxDispVHWASurfBltCompletion(PVBOXDISPDEV pDev, VBOXVHWACMD * pCmd, void * pContext)
+static DECLCALLBACK(void) VBoxDispVHWASurfBltCompletion(PVBOXDISPDEV pDev, VBOXVHWACMD *pCmd, void *pvContext)
 {
+    RT_NOREF(pvContext);
     VBOXVHWACMD_SURF_BLT *pBody = VBOXVHWACMD_BODY(pCmd, VBOXVHWACMD_SURF_BLT);
     PVBOXVHWASURFDESC pSrcDesc = (PVBOXVHWASURFDESC)pBody->SrcGuestSurfInfo;
     PVBOXVHWASURFDESC pDestDesc = (PVBOXVHWASURFDESC)pBody->DstGuestSurfInfo;
@@ -32,8 +32,9 @@ static DECLCALLBACK(void) VBoxDispVHWASurfBltCompletion(PVBOXDISPDEV pDev, VBOXV
     VBoxDispVHWACommandRelease(pDev, pCmd);
 }
 
-static DECLCALLBACK(void) VBoxDispVHWASurfFlipCompletion(PVBOXDISPDEV pDev, VBOXVHWACMD * pCmd, void * pContext)
+static DECLCALLBACK(void) VBoxDispVHWASurfFlipCompletion(PVBOXDISPDEV pDev, VBOXVHWACMD *pCmd, void *pvContext)
 {
+    RT_NOREF(pvContext);
     VBOXVHWACMD_SURF_FLIP *pBody = VBOXVHWACMD_BODY(pCmd, VBOXVHWACMD_SURF_FLIP);
     PVBOXVHWASURFDESC pCurrDesc = (PVBOXVHWASURFDESC)pBody->CurrGuestSurfInfo;
     PVBOXVHWASURFDESC pTargDesc = (PVBOXVHWASURFDESC)pBody->TargGuestSurfInfo;
@@ -69,20 +70,20 @@ int VBoxDispVHWAUpdateDDHalInfo(PVBOXDISPDEV pDev, DD_HALINFO *pHalInfo)
 
     if (VBOXVHWA_CAP(pDev, VBOXVHWA_CAPS_BLT) && VBOXVHWA_CAP(pDev, VBOXVHWA_CAPS_BLTSTRETCH))
     {
-        pHalInfo->ddCaps.dwFXCaps |= DDFXCAPS_BLTSTRETCHX|DDFXCAPS_BLTSTRETCHY|
-                                     DDFXCAPS_BLTSTRETCHXN|DDFXCAPS_BLTSTRETCHYN|
-                                     DDFXCAPS_BLTSHRINKX|DDFXCAPS_BLTSHRINKY|
-                                     DDFXCAPS_BLTSHRINKXN|DDFXCAPS_BLTSHRINKYN|
-                                     DDFXCAPS_BLTARITHSTRETCHY;
+        pHalInfo->ddCaps.dwFXCaps |= DDFXCAPS_BLTSTRETCHX  | DDFXCAPS_BLTSTRETCHY
+                                   | DDFXCAPS_BLTSTRETCHXN | DDFXCAPS_BLTSTRETCHYN
+                                   | DDFXCAPS_BLTSHRINKX   | DDFXCAPS_BLTSHRINKY
+                                   | DDFXCAPS_BLTSHRINKXN  | DDFXCAPS_BLTSHRINKYN
+                                   | DDFXCAPS_BLTARITHSTRETCHY;
     }
 
     if (VBOXVHWA_CAP(pDev, VBOXVHWA_CAPS_OVERLAY) && VBOXVHWA_CAP(pDev, VBOXVHWA_CAPS_OVERLAYSTRETCH))
     {
-        pHalInfo->ddCaps.dwFXCaps |= DDFXCAPS_OVERLAYSTRETCHX|DDFXCAPS_OVERLAYSTRETCHY|
-                                     DDFXCAPS_OVERLAYSTRETCHXN|DDFXCAPS_OVERLAYSTRETCHYN|
-                                     DDFXCAPS_OVERLAYSHRINKX|DDFXCAPS_OVERLAYSHRINKY|
-                                     DDFXCAPS_OVERLAYSHRINKXN|DDFXCAPS_OVERLAYSHRINKYN|
-                                     DDFXCAPS_OVERLAYARITHSTRETCHY;
+        pHalInfo->ddCaps.dwFXCaps |= DDFXCAPS_OVERLAYSTRETCHX  | DDFXCAPS_OVERLAYSTRETCHY
+                                   | DDFXCAPS_OVERLAYSTRETCHXN | DDFXCAPS_OVERLAYSTRETCHYN
+                                   | DDFXCAPS_OVERLAYSHRINKX   | DDFXCAPS_OVERLAYSHRINKY
+                                   | DDFXCAPS_OVERLAYSHRINKXN  | DDFXCAPS_OVERLAYSHRINKYN
+                                   | DDFXCAPS_OVERLAYARITHSTRETCHY;
     }
 
     pHalInfo->ddCaps.dwCKeyCaps = VBoxDispVHWAToDDCKEYCAPS(pDev->vhwa.colorKeyCaps);
@@ -235,8 +236,8 @@ DWORD APIENTRY VBoxDispDDBlt(PDD_BLTDATA lpBlt)
             VBoxDispVHWAFromRECTL(&pBody->u.in.dstRect, &lpBlt->rDest);
             pBody->u.in.hSrcSurf = pSrcDesc->hHostHandle;
             VBoxDispVHWAFromRECTL(&pBody->u.in.srcRect, &lpBlt->rSrc);
-            pBody->DstGuestSurfInfo = (uint64_t)pDstDesc;
-            pBody->SrcGuestSurfInfo = (uint64_t)pSrcDesc;
+            pBody->DstGuestSurfInfo = (uintptr_t)pDstDesc;
+            pBody->SrcGuestSurfInfo = (uintptr_t)pSrcDesc;
 
             pBody->u.in.flags = VBoxDispVHWAFromDDBLTs(lpBlt->dwFlags);
             VBoxDispVHWAFromDDBLTFX(&pBody->u.in.desc, &lpBlt->bltFX);
@@ -313,8 +314,8 @@ DWORD APIENTRY VBoxDispDDFlip(PDD_FLIPDATA lpFlip)
 
             pBody->u.in.hTargSurf = pTargDesc->hHostHandle;
             pBody->u.in.hCurrSurf = pCurrDesc->hHostHandle;
-            pBody->TargGuestSurfInfo = (uint64_t)pTargDesc;
-            pBody->CurrGuestSurfInfo = (uint64_t)pCurrDesc;
+            pBody->TargGuestSurfInfo = (uintptr_t)pTargDesc;
+            pBody->CurrGuestSurfInfo = (uintptr_t)pCurrDesc;
 
             pTargDesc->bVisible = pCurrDesc->bVisible;
             pCurrDesc->bVisible = false;
