@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (C) 2011-2012 Oracle Corporation
+ * Copyright (C) 2011-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -55,23 +55,23 @@ DECLINLINE(void) vbvaVhwaCommandRelease(PVBOXMP_DEVEXT pDevExt, VBOXVHWACMD* pCm
     }
 }
 
-DECLINLINE(void) vbvaVhwaCommandRetain(PVBOXMP_DEVEXT pDevExt, VBOXVHWACMD* pCmd)
+DECLINLINE(void) vbvaVhwaCommandRetain(VBOXVHWACMD *pCmd)
 {
     ASMAtomicIncU32(&pCmd->cRefs);
 }
 
 /* do not wait for completion */
-void vboxVhwaCommandSubmitAsynch(PVBOXMP_DEVEXT pDevExt, VBOXVHWACMD* pCmd, PFNVBOXVHWACMDCOMPLETION pfnCompletion, void * pContext)
+void vboxVhwaCommandSubmitAsynch(PVBOXMP_DEVEXT pDevExt, VBOXVHWACMD* pCmd, PFNVBOXVHWACMDCOMPLETION pfnCompletion, void *pContext)
 {
     pCmd->GuestVBVAReserved1 = (uintptr_t)pfnCompletion;
     pCmd->GuestVBVAReserved2 = (uintptr_t)pContext;
-    vbvaVhwaCommandRetain(pDevExt, pCmd);
+    vbvaVhwaCommandRetain(pCmd);
 
     VBoxHGSMIBufferSubmit(&VBoxCommonFromDeviceExt(pDevExt)->guestCtx, pCmd);
 
-    if(!(pCmd->Flags & VBOXVHWACMD_FLAG_HG_ASYNCH)
-            || ((pCmd->Flags & VBOXVHWACMD_FLAG_GH_ASYNCH_NOCOMPLETION)
-                    && (pCmd->Flags & VBOXVHWACMD_FLAG_HG_ASYNCH_RETURNED)))
+    if(   !(pCmd->Flags & VBOXVHWACMD_FLAG_HG_ASYNCH)
+       || (   (pCmd->Flags & VBOXVHWACMD_FLAG_GH_ASYNCH_NOCOMPLETION)
+           && (pCmd->Flags & VBOXVHWACMD_FLAG_HG_ASYNCH_RETURNED) ) )
     {
         /* the command is completed */
         pfnCompletion(pDevExt, pCmd, pContext);
@@ -82,6 +82,7 @@ void vboxVhwaCommandSubmitAsynch(PVBOXMP_DEVEXT pDevExt, VBOXVHWACMD* pCmd, PFNV
 
 static DECLCALLBACK(void) vboxVhwaCompletionSetEvent(PVBOXMP_DEVEXT pDevExt, VBOXVHWACMD * pCmd, void * pvContext)
 {
+    RT_NOREF(pDevExt,  pCmd);
     RTSemEventSignal((RTSEMEVENT)pvContext);
 }
 
@@ -94,7 +95,7 @@ void vboxVhwaCommandSubmitAsynchByEvent(PVBOXMP_DEVEXT pDevExt, VBOXVHWACMD* pCm
 void vboxVhwaCommandCheckCompletion(PVBOXMP_DEVEXT pDevExt)
 {
     NTSTATUS Status = vboxWddmCallIsr(pDevExt);
-    Assert(Status == STATUS_SUCCESS);
+    AssertNtStatusSuccess(Status);
 }
 
 VBOXVHWACMD* vboxVhwaCommandCreate(PVBOXMP_DEVEXT pDevExt, D3DDDI_VIDEO_PRESENT_SOURCE_ID srcId, VBOXVHWACMD_TYPE enmCmd, VBOXVHWACMD_LENGTH cbCmd)
@@ -183,8 +184,9 @@ int vboxVhwaCommandSubmit(PVBOXMP_DEVEXT pDevExt, VBOXVHWACMD* pCmd)
 }
 
 #ifndef VBOXVHWA_WITH_SHGSMI
-static DECLCALLBACK(void) vboxVhwaCompletionFreeCmd(PVBOXMP_DEVEXT pDevExt, VBOXVHWACMD * pCmd, void * pContext)
+static DECLCALLBACK(void) vboxVhwaCompletionFreeCmd(PVBOXMP_DEVEXT pDevExt, VBOXVHWACMD *pCmd, void *pvContext)
 {
+    RT_NOREF(pvContext);
     vboxVhwaCommandFree(pDevExt, pCmd);
 }
 
@@ -384,13 +386,13 @@ static void vboxVhwaInitSrc(PVBOXMP_DEVEXT pDevExt, D3DDDI_VIDEO_PRESENT_SOURCE_
                     if (pInfo1->u.out.colorKeyCaps & VBOXVHWA_CKEYCAPS_SRCOVERLAY)
                     {
                         pSettings->fFlags |= VBOXVHWA_F_CKEY_SRC;
-                        /* todo: VBOXVHWA_CKEYCAPS_SRCOVERLAYONEACTIVE ? */
+                        /** @todo VBOXVHWA_CKEYCAPS_SRCOVERLAYONEACTIVE ? */
                     }
 
                     if (pInfo1->u.out.colorKeyCaps & VBOXVHWA_CKEYCAPS_DESTOVERLAY)
                     {
                         pSettings->fFlags |= VBOXVHWA_F_CKEY_DST;
-                        /* todo: VBOXVHWA_CKEYCAPS_DESTOVERLAYONEACTIVE ? */
+                        /** @todo VBOXVHWA_CKEYCAPS_DESTOVERLAYONEACTIVE ? */
                     }
                 }
 
@@ -532,9 +534,10 @@ int vboxVhwaHlpDestroySurface(PVBOXMP_DEVEXT pDevExt, PVBOXWDDM_ALLOCATION pSurf
 }
 
 int vboxVhwaHlpPopulateSurInfo(VBOXVHWA_SURFACEDESC *pInfo, PVBOXWDDM_ALLOCATION pSurf,
-        uint32_t fFlags, uint32_t cBackBuffers, uint32_t fSCaps,
-        D3DDDI_VIDEO_PRESENT_SOURCE_ID VidPnSourceId)
+                               uint32_t fFlags, uint32_t cBackBuffers, uint32_t fSCaps,
+                               D3DDDI_VIDEO_PRESENT_SOURCE_ID VidPnSourceId)
 {
+    RT_NOREF(VidPnSourceId);
     memset(pInfo, 0, sizeof(VBOXVHWA_SURFACEDESC));
 
 #if 0
@@ -573,7 +576,7 @@ int vboxVhwaHlpPopulateSurInfo(VBOXVHWA_SURFACEDESC *pInfo, PVBOXWDDM_ALLOCATION
     else
         pInfo->cBackBuffers = 0;
     pInfo->Reserved = 0;
-        /* @todo: color keys */
+        /** @todo color keys */
 //                        pInfo->DstOverlayCK;
 //                        pInfo->DstBltCK;
 //                        pInfo->SrcOverlayCK;
@@ -603,7 +606,7 @@ int vboxVhwaHlpCheckApplySurfInfo(PVBOXWDDM_ALLOCATION pSurf, VBOXVHWA_SURFACEDE
         Assert(pSurf->AllocData.SurfDesc.cbSize);
         pSurf->AllocData.SurfDesc.pitch = pInfo->pitch;
         Assert(pSurf->AllocData.SurfDesc.pitch);
-        /* @todo: make this properly */
+        /** @todo make this properly */
         pSurf->AllocData.SurfDesc.bpp = pSurf->AllocData.SurfDesc.pitch * 8 / pSurf->AllocData.SurfDesc.width;
         Assert(pSurf->AllocData.SurfDesc.bpp);
     }
@@ -753,7 +756,7 @@ int vboxVhwaHlpCheckInit(PVBOXMP_DEVEXT pDevExt, D3DDDI_VIDEO_PRESENT_SOURCE_ID
         return VERR_NOT_SUPPORTED;
 
     int rc = VINF_SUCCESS;
-    /* @todo: need a better sync */
+    /** @todo need a better sync */
     uint32_t cNew = ASMAtomicIncU32(&pSource->Vhwa.cOverlaysCreated);
     if (cNew == 1)
     {
@@ -796,7 +799,7 @@ int vboxVhwaHlpCheckTerm(PVBOXMP_DEVEXT pDevExt, D3DDDI_VIDEO_PRESENT_SOURCE_ID
 
     Assert(!!(pSource->Vhwa.Settings.fFlags & VBOXVHWA_F_ENABLED));
 
-    /* @todo: need a better sync */
+    /** @todo need a better sync */
     uint32_t cNew = ASMAtomicDecU32(&pSource->Vhwa.cOverlaysCreated);
     int rc = VINF_SUCCESS;
     if (!cNew)
