@@ -1,13 +1,10 @@
 /* $Id: file.c $ */
 /** @file
- *
- * VirtualBox Windows Guest Shared Folders
- *
- * File System Driver file routines
+ * VirtualBox Windows Guest Shared Folders - File System Driver file routines.
  */
 
 /*
- * Copyright (C) 2012-2013 Oracle Corporation
+ * Copyright (C) 2012-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -239,7 +236,9 @@ static NTSTATUS vbsfReadInternal(IN PRX_CONTEXT RxContext)
 
     int vboxRC;
 
+#ifdef LOG_ENABLED
     BOOLEAN AsyncIo = BooleanFlagOn(RxContext->Flags, RX_CONTEXT_FLAG_ASYNC_OPERATION);
+#endif
     LONGLONG FileSize;
 
     RxGetFileSizeWithLock((PFCB)capFcb, &FileSize);
@@ -251,7 +250,7 @@ static NTSTATUS vbsfReadInternal(IN PRX_CONTEXT RxContext)
     Log(("VBOXSF: vbsfReadInternal: ByteCount 0x%X, ByteOffset 0x%RX64, FileSize 0x%RX64\n",
          ByteCount, ByteOffset, FileSize));
 
-    /* @todo check if this is necessary. */
+    /** @todo check if this is necessary. */
 #ifdef FCB_STATE_READCACHING_ENABLED    /* Correct spelling for Vista 6001 SDK. */
     if (!FlagOn(capFcb->FcbState, FCB_STATE_READCACHING_ENABLED))
 #else
@@ -268,7 +267,7 @@ static NTSTATUS vbsfReadInternal(IN PRX_CONTEXT RxContext)
             ByteCount = (ULONG)(FileSize - ByteOffset);
     }
 
-    /* @todo read 0 bytes == always success? */
+    /** @todo read 0 bytes == always success? */
     if (   !BufferMdl
         || ByteCount == 0)
     {
@@ -359,7 +358,9 @@ static NTSTATUS vbsfWriteInternal(IN PRX_CONTEXT RxContext)
 
     int vboxRC;
 
+#ifdef LOG_ENABLED
     BOOLEAN AsyncIo = BooleanFlagOn(RxContext->Flags, RX_CONTEXT_FLAG_ASYNC_OPERATION);
+#endif
     LONGLONG FileSize;
 
     RxGetFileSizeWithLock((PFCB)capFcb, &FileSize);
@@ -371,7 +372,7 @@ static NTSTATUS vbsfWriteInternal(IN PRX_CONTEXT RxContext)
     Log(("VBOXSF: vbsfWriteInternal: ByteCount is 0x%X, ByteOffset is 0x%RX64, FileSize 0x%RX64\n",
          ByteCount, ByteOffset, FileSize));
 
-    /* @todo allow to write 0 bytes. */
+    /** @todo allow to write 0 bytes. */
     if (   !BufferMdl
         || ByteCount == 0)
     {
@@ -467,7 +468,7 @@ NTSTATUS VBoxMRxLocks(IN PRX_CONTEXT RxContext)
             return STATUS_NOT_IMPLEMENTED;
 
         case LOWIO_OP_UNLOCK_MULTIPLE:
-            /* @todo Remove multiple locks listed in LowIoContext.ParamsFor.Locks.LockList. */
+            /** @todo Remove multiple locks listed in LowIoContext.ParamsFor.Locks.LockList. */
             Log(("VBOXSF: MRxLocks: Unsupported LOWIO_OP_UNLOCK_MULTIPLE!\n",
                  LowIoContext->Operation));
             return STATUS_NOT_IMPLEMENTED;
@@ -499,10 +500,10 @@ NTSTATUS VBoxMRxLocks(IN PRX_CONTEXT RxContext)
     return Status;
 }
 
-NTSTATUS VBoxMRxCompleteBufferingStateChangeRequest(IN OUT PRX_CONTEXT RxContext,
-                                                    IN OUT PMRX_SRV_OPEN SrvOpen,
-                                                    IN PVOID pContext)
+NTSTATUS VBoxMRxCompleteBufferingStateChangeRequest(IN OUT PRX_CONTEXT RxContext, IN OUT PMRX_SRV_OPEN SrvOpen,
+                                                    IN PVOID pvContext)
 {
+    RT_NOREF(RxContext, SrvOpen, pvContext);
     Log(("VBOXSF: MRxCompleteBufferingStateChangeRequest: not implemented\n"));
     return STATUS_NOT_IMPLEMENTED;
 }
@@ -586,10 +587,17 @@ NTSTATUS vbsfSetEndOfFile(IN OUT struct _RX_CONTEXT * RxContext,
     return Status;
 }
 
-NTSTATUS VBoxMRxExtendStub(IN OUT struct _RX_CONTEXT * RxContext,
-                           IN OUT PLARGE_INTEGER pNewFileSize,
-                           OUT PLARGE_INTEGER pNewAllocationSize)
+/** See PMRX_EXTENDFILE_CALLDOWN in ddk/mrx.h
+ *
+ * Documentation says it returns STATUS_SUCCESS on success and an error
+ * status on failure, so the ULONG return type is probably just a typo that
+ * stuck.
+ */
+ULONG NTAPI VBoxMRxExtendStub(IN OUT struct _RX_CONTEXT * RxContext, IN OUT PLARGE_INTEGER pNewFileSize,
+                              OUT PLARGE_INTEGER pNewAllocationSize)
 {
+    RT_NOREF(RxContext);
+
     /* Note: On Windows hosts vbsfSetEndOfFile returns ACCESS_DENIED if the file has been
      *       opened in APPEND mode. Writes to a file will extend it anyway, therefore it is
      *       better to not call the host at all and tell the caller that the file was extended.
@@ -601,3 +609,4 @@ NTSTATUS VBoxMRxExtendStub(IN OUT struct _RX_CONTEXT * RxContext,
 
     return STATUS_SUCCESS;
 }
+
