@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2007 knut st. osmundsen <bird-src-spam@anduin.net>
+ * Copyright (C) 2007-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -19,7 +19,7 @@
  *
  * VBoxDrv - OS/2 specifics.
  *
- * Copyright (c) 2007 knut st. osmundsen <bird-src-spam@anduin.net>
+ * Copyright (c) 2007-2012 knut st. osmundsen <bird-src-spam@anduin.net>
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -44,9 +44,9 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include <os2ddk/bsekee.h>
 
 #include "VBoxGuestInternal.h"
@@ -62,9 +62,9 @@
 #include <iprt/param.h>
 
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 /**
  * Device extention & session data association structure.
  */
@@ -105,14 +105,14 @@ extern uint16_t             g_cchInitTextMax;
 RT_C_DECLS_END
 
 
-/*******************************************************************************
-*   Internal Functions                                                         *
-*******************************************************************************/
-static int vboxGuestOS2MapMemory(void);
-static VBOXOSTYPE vboxGuestOS2DetectVersion(void);
+/*********************************************************************************************************************************
+*   Internal Functions                                                                                                           *
+*********************************************************************************************************************************/
+static int vgdrvOS2MapMemory(void);
+static VBOXOSTYPE vgdrvOS2DetectVersion(void);
 
 /* in VBoxGuestA-os2.asm */
-DECLASM(int) VBoxGuestOS2SetIRQ(uint8_t bIRQ);
+DECLASM(int) vgdrvOS2DevHlpSetIRQ(uint8_t bIRQ);
 
 
 /**
@@ -123,9 +123,9 @@ DECLASM(int) VBoxGuestOS2SetIRQ(uint8_t bIRQ);
  * @returns 0 on success, non-zero on failure.
  * @param   pszArgs     Pointer to the device arguments.
  */
-DECLASM(int) VBoxGuestOS2Init(const char *pszArgs)
+DECLASM(int) vgdrvOS2Init(const char *pszArgs)
 {
-    Log(("VBoxGuestOS2Init: pszArgs='%s' MMIO=0x%RX32 IOPort=0x%RX16 Int=%#x Bus=%#x Dev=%#x Fun=%d\n",
+    Log(("vgdrvOS2Init: pszArgs='%s' MMIO=0x%RX32 IOPort=0x%RX16 Int=%#x Bus=%#x Dev=%#x Fun=%d\n",
          pszArgs, g_PhysMMIOBase, g_IOPortBase, g_bInterruptLine, g_bPciBusNo, g_bPciDevFunNo >> 3, g_bPciDevFunNo & 7));
 
     /*
@@ -142,22 +142,20 @@ DECLASM(int) VBoxGuestOS2Init(const char *pszArgs)
         /*
          * Map the MMIO memory if found.
          */
-        rc = vboxGuestOS2MapMemory();
+        rc = vgdrvOS2MapMemory();
         if (RT_SUCCESS(rc))
         {
             /*
              * Initialize the device extension.
              */
             if (g_MemMapMMIO != NIL_RTR0MEMOBJ)
-                rc = VBoxGuestInitDevExt(&g_DevExt, g_IOPortBase,
-                                         RTR0MemObjAddress(g_MemMapMMIO),
-                                         RTR0MemObjSize(g_MemMapMMIO),
-                                         vboxGuestOS2DetectVersion(),
-                                         0);
+                rc = VGDrvCommonInitDevExt(&g_DevExt, g_IOPortBase,
+                                           RTR0MemObjAddress(g_MemMapMMIO),
+                                           RTR0MemObjSize(g_MemMapMMIO),
+                                           vgdrvOS2DetectVersion(),
+                                           0);
             else
-                rc = VBoxGuestInitDevExt(&g_DevExt, g_IOPortBase, NULL, 0,
-                                         vboxGuestOS2DetectVersion(),
-                                         0);
+                rc = VGDrvCommonInitDevExt(&g_DevExt, g_IOPortBase, NULL, 0, vgdrvOS2DetectVersion(), 0);
             if (RT_SUCCESS(rc))
             {
                 /*
@@ -171,10 +169,10 @@ DECLASM(int) VBoxGuestOS2Init(const char *pszArgs)
                      */
                     if (g_bInterruptLine)
                     {
-                        rc = VBoxGuestOS2SetIRQ(g_bInterruptLine);
+                        rc = vgdrvOS2DevHlpSetIRQ(g_bInterruptLine);
                         if (rc)
                         {
-                            Log(("VBoxGuestOS2SetIRQ(%d) -> %d\n", g_bInterruptLine, rc));
+                            Log(("vgdrvOS2DevHlpSetIRQ(%d) -> %d\n", g_bInterruptLine, rc));
                             rc = RTErrConvertFromOS2(rc);
                         }
                     }
@@ -188,10 +186,10 @@ DECLASM(int) VBoxGuestOS2Init(const char *pszArgs)
                             strcpy(&g_szInitText[0],
                                    "\r\n"
                                    "VirtualBox Guest Additions Driver for OS/2 version " VBOX_VERSION_STRING "\r\n"
-                                   "Copyright (C) 2008-2010 Oracle Corporation\r\n");
+                                   "Copyright (C) 2008-" VBOX_C_YEAR " " VBOX_VENDOR "\r\n");
                             g_cchInitText = strlen(&g_szInitText[0]);
                         }
-                        Log(("VBoxGuestOS2Init: Successfully loaded\n%s", g_szInitText));
+                        Log(("vgdrvOS2Init: Successfully loaded\n%s", g_szInitText));
                         return VINF_SUCCESS;
                     }
 
@@ -200,10 +198,10 @@ DECLASM(int) VBoxGuestOS2Init(const char *pszArgs)
                 }
                 else
                     g_cchInitText = RTStrPrintf(&g_szInitText[0], g_cchInitTextMax, "VBoxGuest.sys: RTSpinlockCreate failed, rc=%Rrc\n", rc);
-                VBoxGuestDeleteDevExt(&g_DevExt);
+                VGDrvCommonDeleteDevExt(&g_DevExt);
             }
             else
-                g_cchInitText = RTStrPrintf(&g_szInitText[0], g_cchInitTextMax, "VBoxGuest.sys: VBoxGuestOS2InitDevExt failed, rc=%Rrc\n", rc);
+                g_cchInitText = RTStrPrintf(&g_szInitText[0], g_cchInitTextMax, "VBoxGuest.sys: vgdrvOS2InitDevExt failed, rc=%Rrc\n", rc);
 
             int rc2 = RTR0MemObjFree(g_MemObjMMIO, true /* fFreeMappings */); AssertRC(rc2);
             g_MemObjMMIO = g_MemMapMMIO = NIL_RTR0MEMOBJ;
@@ -215,7 +213,7 @@ DECLASM(int) VBoxGuestOS2Init(const char *pszArgs)
     else
         g_cchInitText = RTStrPrintf(&g_szInitText[0], g_cchInitTextMax, "VBoxGuest.sys: RTR0Init failed, rc=%Rrc\n", rc);
 
-    RTLogBackdoorPrintf("VBoxGuestOS2Init: failed rc=%Rrc - %s", rc, &g_szInitText[0]);
+    RTLogBackdoorPrintf("vgdrvOS2Init: failed rc=%Rrc - %s", rc, &g_szInitText[0]);
     return rc;
 }
 
@@ -225,10 +223,8 @@ DECLASM(int) VBoxGuestOS2Init(const char *pszArgs)
  *
  * @returns VBox status code.
  * @retval  VERR_VERSION_MISMATCH       The VMMDev memory didn't meet our expectations.
- *
- * @param   pDevExt     The device extension.
  */
-static int vboxGuestOS2MapMemory(void)
+static int vgdrvOS2MapMemory(void)
 {
     const RTCCPHYS PhysMMIOBase = g_PhysMMIOBase;
 
@@ -257,8 +253,7 @@ static int vboxGuestOS2MapMemory(void)
     }
     if (RT_FAILURE(rc))
     {
-        Log(("vboxGuestOS2MapMemory: RTR0MemObjEnterPhys(,%RCp,%zx) -> %Rrc\n",
-             PhysMMIOBase, cb, rc));
+        Log(("vgdrvOS2MapMemory: RTR0MemObjEnterPhys(,%RCp,%zx) -> %Rrc\n", PhysMMIOBase, cb, rc));
         return rc;
     }
 
@@ -270,8 +265,7 @@ static int vboxGuestOS2MapMemory(void)
      * And second, because PGM doesn't necessarily respect the cache/writethru bits
      * anyway for normal RAM.
      */
-    rc = RTR0MemObjMapKernel(&g_MemMapMMIO, g_MemObjMMIO, (void *)-1, 0,
-                             RTMEM_PROT_READ | RTMEM_PROT_WRITE);
+    rc = RTR0MemObjMapKernel(&g_MemMapMMIO, g_MemObjMMIO, (void *)-1, 0, RTMEM_PROT_READ | RTMEM_PROT_WRITE);
     if (RT_SUCCESS(rc))
     {
         /*
@@ -289,7 +283,7 @@ static int vboxGuestOS2MapMemory(void)
             if (RT_ALIGN_32(pVMMDev->u32Size, PAGE_SIZE) == cb)
                 return VINF_SUCCESS;
 
-            Log(("vboxGuestOS2MapMemory: Actual size %#RX32 (tried %#zx)\n", pVMMDev->u32Size, cb));
+            Log(("vgdrvOS2MapMemory: Actual size %#RX32 (tried %#zx)\n", pVMMDev->u32Size, cb));
             cb = RT_ALIGN_32(pVMMDev->u32Size, PAGE_SIZE);
 
             rc = RTR0MemObjFree(g_MemObjMMIO, true); AssertRC(rc);
@@ -298,28 +292,24 @@ static int vboxGuestOS2MapMemory(void)
             rc = RTR0MemObjEnterPhys(&g_MemObjMMIO, PhysMMIOBase, cb, RTMEM_CACHE_POLICY_DONT_CARE);
             if (RT_SUCCESS(rc))
             {
-                rc = RTR0MemObjMapKernel(&g_MemMapMMIO, g_MemObjMMIO, (void *)-1, 0,
-                                         RTMEM_PROT_READ | RTMEM_PROT_WRITE);
+                rc = RTR0MemObjMapKernel(&g_MemMapMMIO, g_MemObjMMIO, (void *)-1, 0, RTMEM_PROT_READ | RTMEM_PROT_WRITE);
                 if (RT_SUCCESS(rc))
                     return VINF_SUCCESS;
 
-                Log(("vboxGuestOS2MapMemory: RTR0MemObjMapKernel [%RCp,%zx] -> %Rrc (2nd)\n",
-                     PhysMMIOBase, cb, rc));
+                Log(("vgdrvOS2MapMemory: RTR0MemObjMapKernel [%RCp,%zx] -> %Rrc (2nd)\n", PhysMMIOBase, cb, rc));
             }
             else
-                Log(("vboxGuestOS2MapMemory: RTR0MemObjEnterPhys(,%RCp,%zx) -> %Rrc (2nd)\n",
-                     PhysMMIOBase, cb, rc));
+                Log(("vgdrvOS2MapMemory: RTR0MemObjEnterPhys(,%RCp,%zx) -> %Rrc (2nd)\n", PhysMMIOBase, cb, rc));
         }
         else
         {
             rc = VERR_VERSION_MISMATCH;
-            LogRel(("vboxGuestOS2MapMemory: Bogus VMMDev memory; u32Version=%RX32 (expected %RX32) u32Size=%RX32\n",
+            LogRel(("vgdrvOS2MapMemory: Bogus VMMDev memory; u32Version=%RX32 (expected %RX32) u32Size=%RX32\n",
                     pVMMDev->u32Version, VMMDEV_MEMORY_VERSION, pVMMDev->u32Size));
         }
     }
     else
-        Log(("vboxGuestOS2MapMemory: RTR0MemObjMapKernel [%RCp,%zx] -> %Rrc\n",
-             PhysMMIOBase, cb, rc));
+        Log(("vgdrvOS2MapMemory: RTR0MemObjMapKernel [%RCp,%zx] -> %Rrc\n", PhysMMIOBase, cb, rc));
 
     int rc2 = RTR0MemObjFree(g_MemObjMMIO, true /* fFreeMappings */); AssertRC(rc2);
     g_MemObjMMIO = g_MemMapMMIO = NIL_RTR0MEMOBJ;
@@ -328,11 +318,11 @@ static int vboxGuestOS2MapMemory(void)
 
 
 /**
- * Called fromn VBoxGuestOS2Init to determine which OS/2 version this is.
+ * Called fromn vgdrvOS2Init to determine which OS/2 version this is.
  *
  * @returns VBox OS/2 type.
  */
-static VBOXOSTYPE vboxGuestOS2DetectVersion(void)
+static VBOXOSTYPE vgdrvOS2DetectVersion(void)
 {
     VBOXOSTYPE enmOSType = VBOXOSTYPE_OS2;
 
@@ -352,7 +342,7 @@ static VBOXOSTYPE vboxGuestOS2DetectVersion(void)
 }
 
 
-DECLASM(int) VBoxGuestOS2Open(uint16_t sfn)
+DECLASM(int) vgdrvOS2Open(uint16_t sfn)
 {
     int                 rc;
     PVBOXGUESTSESSION   pSession;
@@ -360,7 +350,7 @@ DECLASM(int) VBoxGuestOS2Open(uint16_t sfn)
     /*
      * Create a new session.
      */
-    rc = VBoxGuestCreateUserSession(&g_DevExt, &pSession);
+    rc = VGDrvCommonCreateUserSession(&g_DevExt, &pSession);
     if (RT_SUCCESS(rc))
     {
         pSession->sfn = sfn;
@@ -372,17 +362,17 @@ DECLASM(int) VBoxGuestOS2Open(uint16_t sfn)
         RTSpinlockAcquire(g_Spinlock);
         pSession->pNextHash = g_apSessionHashTab[iHash];
         g_apSessionHashTab[iHash] = pSession;
-        RTSpinlockReleaseNoInts(g_Spinlock);
+        RTSpinlockRelease(g_Spinlock);
     }
 
-    Log(("VBoxGuestOS2Open: g_DevExt=%p pSession=%p rc=%d pid=%d\n", &g_DevExt, pSession, rc, (int)RTProcSelf()));
+    Log(("vgdrvOS2Open: g_DevExt=%p pSession=%p rc=%d pid=%d\n", &g_DevExt, pSession, rc, (int)RTProcSelf()));
     return rc;
 }
 
 
-DECLASM(int) VBoxGuestOS2Close(uint16_t sfn)
+DECLASM(int) vgdrvOS2Close(uint16_t sfn)
 {
-    Log(("VBoxGuestOS2Close: pid=%d sfn=%d\n", (int)RTProcSelf(), sfn));
+    Log(("vgdrvOS2Close: pid=%d sfn=%d\n", (int)RTProcSelf(), sfn));
 
     /*
      * Remove from the hash table.
@@ -421,7 +411,7 @@ DECLASM(int) VBoxGuestOS2Close(uint16_t sfn)
             }
         }
     }
-    RTSpinlockReleaseNoInts(g_Spinlock);
+    RTSpinlockRelease(g_Spinlock);
     if (!pSession)
     {
         Log(("VBoxGuestIoctl: WHUT?!? pSession == NULL! This must be a mistake... pid=%d sfn=%d\n", (int)Process, sfn));
@@ -431,12 +421,12 @@ DECLASM(int) VBoxGuestOS2Close(uint16_t sfn)
     /*
      * Close the session.
      */
-    VBoxGuestCloseSession(&g_DevExt, pSession);
+    VGDrvCommonCloseSession(&g_DevExt, pSession);
     return 0;
 }
 
 
-DECLASM(int) VBoxGuestOS2IOCtlFast(uint16_t sfn, uint8_t iFunction, int32_t *prc)
+DECLASM(int) vgdrvOS2IOCtlFast(uint16_t sfn, uint8_t iFunction, int32_t *prc)
 {
     /*
      * Find the session.
@@ -454,7 +444,7 @@ DECLASM(int) VBoxGuestOS2IOCtlFast(uint16_t sfn, uint8_t iFunction, int32_t *prc
                &&   (   pSession->sfn != sfn
                      || pSession->Process != Process));
     }
-    RTSpinlockReleaseNoInts(g_Spinlock);
+    RTSpinlockRelease(g_Spinlock);
     if (RT_UNLIKELY(!pSession))
     {
         Log(("VBoxGuestIoctl: WHAT?!? pSession == NULL! This must be a mistake... pid=%d\n", (int)Process));
@@ -464,7 +454,7 @@ DECLASM(int) VBoxGuestOS2IOCtlFast(uint16_t sfn, uint8_t iFunction, int32_t *prc
     /*
      * Dispatch the fast IOCtl.
      */
-    *prc = VBoxGuestCommonIOCtlFast(iFunction, &g_DevExt, pSession);
+    *prc = VGDrvCommonIoCtlFast(iFunction, &g_DevExt, pSession);
     return 0;
 }
 
@@ -485,7 +475,7 @@ DECLASM(int) VBoxGuestOS2IOCtlFast(uint16_t sfn, uint8_t iFunction, int32_t *prc
  *
  * @remark  This is called from the 16-bit thunker as well as directly from the 32-bit clients.
  */
-DECLASM(int) VBoxGuestOS2IDCService(uint32_t u32Session, unsigned iFunction, void *pvData, size_t cbData, size_t *pcbDataReturned)
+DECLASM(int) VGDrvOS2IDCService(uint32_t u32Session, unsigned iFunction, void *pvData, size_t cbData, size_t *pcbDataReturned)
 {
     PVBOXGUESTSESSION pSession = (PVBOXGUESTSESSION)u32Session;
     AssertPtrReturn(pSession, VERR_INVALID_POINTER);
@@ -496,12 +486,12 @@ DECLASM(int) VBoxGuestOS2IDCService(uint32_t u32Session, unsigned iFunction, voi
     switch (iFunction)
     {
         default:
-            rc = VBoxGuestCommonIOCtl(iFunction, &g_DevExt, pSession, pvData, cbData, pcbDataReturned);
+            rc = VGDrvCommonIoCtl(iFunction, &g_DevExt, pSession, pvData, cbData, pcbDataReturned);
             break;
 
         case VBOXGUEST_IOCTL_OS2_IDC_DISCONNECT:
             pSession->sfn = 0;
-            VBoxGuestCloseSession(&g_DevExt, pSession);
+            VGDrvCommonCloseSession(&g_DevExt, pSession);
             rc = VINF_SUCCESS;
             break;
     }
@@ -514,10 +504,10 @@ DECLASM(int) VBoxGuestOS2IDCService(uint32_t u32Session, unsigned iFunction, voi
  *
  * @returns Pointer to the session.
  */
-DECLASM(PVBOXGUESTSESSION) VBoxGuestOS2IDCConnect(void)
+DECLASM(PVBOXGUESTSESSION) vgdrvOS2IDCConnect(void)
 {
     PVBOXGUESTSESSION pSession;
-    int rc = VBoxGuestCreateKernelSession(&g_DevExt, &pSession);
+    int rc = VGDrvCommonCreateKernelSession(&g_DevExt, &pSession);
     if (RT_SUCCESS(rc))
     {
         pSession->sfn = 0xffff;
@@ -527,7 +517,8 @@ DECLASM(PVBOXGUESTSESSION) VBoxGuestOS2IDCConnect(void)
 }
 
 
-DECLASM(int) VBoxGuestOS2IOCtl(uint16_t sfn, uint8_t iCat, uint8_t iFunction, void *pvParm, void *pvData, uint16_t *pcbParm, uint16_t *pcbData)
+DECLASM(int) vgdrvOS2IOCtl(uint16_t sfn, uint8_t iCat, uint8_t iFunction, void *pvParm, void *pvData,
+                           uint16_t *pcbParm, uint16_t *pcbData)
 {
     /*
      * Find the session.
@@ -545,7 +536,7 @@ DECLASM(int) VBoxGuestOS2IOCtl(uint16_t sfn, uint8_t iCat, uint8_t iFunction, vo
                &&   (   pSession->sfn != sfn
                      || pSession->Process != Process));
     }
-    RTSpinlockReleaseNoInts(g_Spinlock);
+    RTSpinlockRelease(g_Spinlock);
     if (!pSession)
     {
         Log(("VBoxGuestIoctl: WHAT?!? pSession == NULL! This must be a mistake... pid=%d\n", (int)Process));
@@ -563,7 +554,7 @@ DECLASM(int) VBoxGuestOS2IOCtl(uint16_t sfn, uint8_t iCat, uint8_t iFunction, vo
      */
     if (RT_LIKELY(iCat == VBOXGUEST_IOCTL_CATEGORY))
     {
-        Log(("VBoxGuestOS2IOCtl: pSession=%p iFunction=%#x pvParm=%p pvData=%p *pcbParm=%d *pcbData=%d\n", pSession, iFunction, pvParm, pvData, *pcbParm, *pcbData));
+        Log(("vgdrvOS2IOCtl: pSession=%p iFunction=%#x pvParm=%p pvData=%p *pcbParm=%d *pcbData=%d\n", pSession, iFunction, pvParm, pvData, *pcbParm, *pcbData));
         Assert(pvParm || !*pcbData);
         Assert(pvData);
         Assert(*pcbData == sizeof(int32_t)); /* the return code */
@@ -599,7 +590,7 @@ DECLASM(int) VBoxGuestOS2IOCtl(uint16_t sfn, uint8_t iCat, uint8_t iFunction, vo
          * Process the IOCtl.
          */
         size_t cbDataReturned;
-        rc = VBoxGuestCommonIOCtl(iFunction, &g_DevExt, pSession, pvParm, *pcbParm, &cbDataReturned);
+        rc = VGDrvCommonIoCtl(iFunction, &g_DevExt, pSession, pvParm, *pcbParm, &cbDataReturned);
 
         /*
          * Unlock the buffers.
@@ -622,7 +613,7 @@ DECLASM(int) VBoxGuestOS2IOCtl(uint16_t sfn, uint8_t iCat, uint8_t iFunction, vo
         AssertMsgReturn(!rc, ("KernCopyOut(%p, %p, sizeof(int32_t)) -> %d\n", pvData, &rc, rc), VERR_LOCK_FAILED);
 #endif
 
-        Log2(("VBoxGuestOS2IOCtl: returns VINF_SUCCESS / %d\n", rc));
+        Log2(("vgdrvOS2IOCtl: returns VINF_SUCCESS / %d\n", rc));
         return VINF_SUCCESS;
     }
     return VERR_NOT_SUPPORTED;
@@ -634,15 +625,15 @@ DECLASM(int) VBoxGuestOS2IOCtl(uint16_t sfn, uint8_t iCat, uint8_t iFunction, vo
  *
  * @returns true if it's our interrupt, false it isn't.
  */
-DECLASM(bool) VBoxGuestOS2ISR(void)
+DECLASM(bool) vgdrvOS2ISR(void)
 {
-    Log(("VBoxGuestOS2ISR\n"));
+    Log(("vgdrvOS2ISR\n"));
 
-    return VBoxGuestCommonISR(&g_DevExt);
+    return VGDrvCommonISR(&g_DevExt);
 }
 
 
-void VBoxGuestNativeISRMousePollEvent(PVBOXGUESTDEVEXT pDevExt)
+void VGDrvNativeISRMousePollEvent(PVBOXGUESTDEVEXT pDevExt)
 {
     /* No polling on OS/2 */
     NOREF(pDevExt);
@@ -659,7 +650,7 @@ void VBoxGuestNativeISRMousePollEvent(PVBOXGUESTDEVEXT pDevExt)
  * @param   pachChars   Pointer to an array of utf-8 characters.
  * @param   cbChars     Number of bytes in the character array pointed to by pachChars.
  */
-static DECLCALLBACK(size_t) vboxGuestNativeLogOutput(void *pvArg, const char *pachChars, size_t cbChars)
+static DECLCALLBACK(size_t) vgdrvOS2LogOutput(void *pvArg, const char *pachChars, size_t cbChars)
 {
     size_t cchWritten = 0;
     while (cbChars-- > 0)
@@ -687,7 +678,7 @@ int SUPR0Printf(const char *pszFormat, ...)
 #endif
 
     va_start(va, pszFormat);
-    int cch = RTLogFormatV(vboxGuestNativeLogOutput, NULL, pszFormat, va);
+    int cch = RTLogFormatV(vgdrvOS2LogOutput, NULL, pszFormat, va);
     va_end(va);
 
     return cch;
