@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -54,17 +54,18 @@
 typedef struct NULLAUDIOSTREAMOUT
 {
     /** Note: Always must come first! */
-    PDMAUDIOHSTSTRMOUT hw;
-    uint64_t u64TicksLast;
-    uint64_t csPlayBuffer;
-    uint8_t *pu8PlayBuffer;
-} NULLAUDIOSTREAMOUT;
+    PDMAUDIOHSTSTRMOUT streamOut;
+    uint64_t           u64TicksLast;
+    uint64_t           csPlayBuffer;
+    uint8_t           *pu8PlayBuffer;
+} NULLAUDIOSTREAMOUT, *PNULLAUDIOSTREAMOUT;
 
 typedef struct NULLAUDIOSTREAMIN
 {
     /** Note: Always must come first! */
-    PDMAUDIOHSTSTRMIN hw;
-} NULLAUDIOSTREAMIN;
+    PDMAUDIOHSTSTRMIN  streamIn;
+    uint64_t           u64TicksLast;
+} NULLAUDIOSTREAMIN, *PNULLAUDIOSTREAMIN;
 
 /**
  * NULL audio driver instance data.
@@ -103,15 +104,16 @@ static DECLCALLBACK(int) drvHostNullAudioInit(PPDMIHOSTAUDIO pInterface)
 }
 
 static DECLCALLBACK(int) drvHostNullAudioInitIn(PPDMIHOSTAUDIO pInterface,
-                                                PPDMAUDIOHSTSTRMIN pHstStrmIn, PPDMAUDIOSTREAMCFG pCfg,
-                                                PDMAUDIORECSOURCE enmRecSource,
-                                                uint32_t *pcSamples)
+                                                PPDMAUDIOHSTSTRMIN pHstStrmIn,
+                                                PPDMAUDIOSTREAMCFG pCfgReq, PPDMAUDIOSTREAMCFG pCfgAcq,
+                                                PDMAUDIORECSOURCE enmRecSource, uint32_t *pcSamples)
 {
     NOREF(pInterface);
     NOREF(enmRecSource);
+    NOREF(pCfgAcq);
 
     /* Just adopt the wanted stream configuration. */
-    int rc = drvAudioStreamCfgToProps(pCfg, &pHstStrmIn->Props);
+    int rc = DrvAudioStreamCfgToProps(pCfgReq, &pHstStrmIn->Props);
     if (RT_SUCCESS(rc))
     {
         if (pcSamples)
@@ -122,18 +124,20 @@ static DECLCALLBACK(int) drvHostNullAudioInitIn(PPDMIHOSTAUDIO pInterface,
 }
 
 static DECLCALLBACK(int) drvHostNullAudioInitOut(PPDMIHOSTAUDIO pInterface,
-                                                 PPDMAUDIOHSTSTRMOUT pHstStrmOut, PPDMAUDIOSTREAMCFG pCfg,
+                                                 PPDMAUDIOHSTSTRMOUT pHstStrmOut,
+                                                 PPDMAUDIOSTREAMCFG pCfgReq, PPDMAUDIOSTREAMCFG pCfgAcq,
                                                  uint32_t *pcSamples)
 {
     NOREF(pInterface);
+    NOREF(pCfgAcq);
 
     /* Just adopt the wanted stream configuration. */
-    int rc = drvAudioStreamCfgToProps(pCfg, &pHstStrmOut->Props);
+    int rc = DrvAudioStreamCfgToProps(pCfgReq, &pHstStrmOut->Props);
     if (RT_SUCCESS(rc))
     {
-        NULLAUDIOSTREAMOUT *pNullStrmOut = (NULLAUDIOSTREAMOUT *)pHstStrmOut;
-        pNullStrmOut->u64TicksLast = 0;
-        pNullStrmOut->csPlayBuffer = _1K;
+        PNULLAUDIOSTREAMOUT pNullStrmOut = (PNULLAUDIOSTREAMOUT)pHstStrmOut;
+        pNullStrmOut->u64TicksLast  = 0;
+        pNullStrmOut->csPlayBuffer  = _1K;
         pNullStrmOut->pu8PlayBuffer = (uint8_t *)RTMemAlloc(_1K << pHstStrmOut->Props.cShift);
         if (pNullStrmOut->pu8PlayBuffer)
         {
@@ -160,18 +164,19 @@ static DECLCALLBACK(int) drvHostNullAudioPlayOut(PPDMIHOSTAUDIO pInterface, PPDM
                                                  uint32_t *pcSamplesPlayed)
 {
     PDRVHOSTNULLAUDIO pDrv = RT_FROM_MEMBER(pInterface, DRVHOSTNULLAUDIO, IHostAudio);
-    NULLAUDIOSTREAMOUT *pNullStrmOut = (NULLAUDIOSTREAMOUT *)pHstStrmOut;
+    PNULLAUDIOSTREAMOUT pNullStrmOut = (PNULLAUDIOSTREAMOUT)pHstStrmOut;
 
     /* Consume as many samples as would be played at the current frequency since last call. */
-    uint32_t csLive = drvAudioHstOutSamplesLive(pHstStrmOut);
-    uint64_t u64TicksNow = PDMDrvHlpTMGetVirtualTime(pDrv->pDrvIns);
+    uint32_t csLive          = AudioMixBufLive(&pHstStrmOut->MixBuf);
+    uint64_t u64TicksNow     = PDMDrvHlpTMGetVirtualTime(pDrv->pDrvIns);
     uint64_t u64TicksElapsed = u64TicksNow  - pNullStrmOut->u64TicksLast;
-    uint64_t u64TicksFreq = PDMDrvHlpTMGetVirtualFreq(pDrv->pDrvIns);
+    uint64_t u64TicksFreq    = PDMDrvHlpTMGetVirtualFreq(pDrv->pDrvIns);
 
     /* Remember when samples were consumed. */
     pNullStrmOut->u64TicksLast = u64TicksNow;
 
-    /* Minimize the rounding error by adding 0.5: samples = int((u64TicksElapsed * samplesFreq) / u64TicksFreq + 0.5).
+    /*
+     * Minimize the rounding error by adding 0.5: samples = int((u64TicksElapsed * samplesFreq) / u64TicksFreq + 0.5).
      * If rounding is not taken into account then the playback rate will be consistently lower that expected.
      */
     uint64_t cSamplesPlayed = (2 * u64TicksElapsed * pHstStrmOut->Props.uHz + u64TicksFreq) / u64TicksFreq / 2;
@@ -183,7 +188,8 @@ static DECLCALLBACK(int) drvHostNullAudioPlayOut(PPDMIHOSTAUDIO pInterface, PPDM
     cSamplesPlayed = RT_MIN(cSamplesPlayed, pNullStrmOut->csPlayBuffer);
 
     uint32_t csRead = 0;
-    AudioMixBufReadCirc(&pHstStrmOut->MixBuf, pNullStrmOut->pu8PlayBuffer, cSamplesPlayed << pHstStrmOut->Props.cShift, &csRead);
+    AudioMixBufReadCirc(&pHstStrmOut->MixBuf, pNullStrmOut->pu8PlayBuffer,
+                        AUDIOMIXBUF_S2B(&pHstStrmOut->MixBuf, cSamplesPlayed), &csRead);
     AudioMixBufFinish(&pHstStrmOut->MixBuf, csRead);
 
     if (pcSamplesPlayed)
@@ -195,11 +201,55 @@ static DECLCALLBACK(int) drvHostNullAudioPlayOut(PPDMIHOSTAUDIO pInterface, PPDM
 static DECLCALLBACK(int) drvHostNullAudioCaptureIn(PPDMIHOSTAUDIO pInterface, PPDMAUDIOHSTSTRMIN pHstStrmIn,
                                                    uint32_t *pcSamplesCaptured)
 {
-    /* Never capture anything. */
-    if (pcSamplesCaptured)
-        *pcSamplesCaptured = 0;
+    PDRVHOSTNULLAUDIO pDrv = RT_FROM_MEMBER(pInterface, DRVHOSTNULLAUDIO, IHostAudio);
+    PNULLAUDIOSTREAMIN pNullStrmIn = (PNULLAUDIOSTREAMIN)pHstStrmIn;
 
-    return VINF_SUCCESS;
+    /* Consume as many samples as would have been captured at the current frequency since last call. */
+    uint64_t u64TicksNow     = PDMDrvHlpTMGetVirtualTime(pDrv->pDrvIns);
+    uint64_t u64TicksElapsed = u64TicksNow  - pNullStrmIn->u64TicksLast;
+    uint64_t u64TicksFreq    = PDMDrvHlpTMGetVirtualFreq(pDrv->pDrvIns);
+
+    /* Remember when samples were consumed. */
+    pNullStrmIn->u64TicksLast = u64TicksNow;
+
+    /*
+     * Minimize the rounding error by adding 0.5: samples = int((u64TicksElapsed * samplesFreq) / u64TicksFreq + 0.5).
+     * If rounding is not taken into account then the capturing rate will be consistently lower that expected.
+     */
+    uint64_t cbCaptured = AUDIOMIXBUF_S2B(&pHstStrmIn->MixBuf,
+                                          (2 * u64TicksElapsed * pHstStrmIn->Props.uHz + u64TicksFreq) / u64TicksFreq / 2);
+    AssertPtr(pHstStrmIn->pGstStrmIn);
+    uint32_t cbFree     = AudioMixBufFreeBytes(&pHstStrmIn->pGstStrmIn->MixBuf);
+
+    /* Don't capture more than free (available). */
+    if (cbCaptured > cbFree)
+        cbCaptured = cbFree;
+
+    /* Create NULL audio input data on-the-fly. */
+    size_t   cbBuf = cbCaptured;
+    AssertReturn(cbBuf, VERR_INVALID_PARAMETER);
+    uint8_t *paBuf = (uint8_t *)RTMemAllocZ(cbBuf);
+    AssertPtrReturn(paBuf, VERR_NO_MEMORY);
+
+    uint32_t csCapturedTotal = 0;
+
+    uint32_t csWritten;
+    int rc = AudioMixBufWriteAt(&pHstStrmIn->MixBuf, 0 /* Offset */, paBuf, (uint32_t)cbBuf, &csWritten);
+    if (RT_SUCCESS(rc))
+    {
+        uint32_t csMixed = 0;
+        rc = AudioMixBufMixToParentEx(&pHstStrmIn->MixBuf, 0 /* Offset */, csWritten, &csMixed);
+        if (RT_SUCCESS(rc))
+            csCapturedTotal = csMixed;
+    }
+
+    RTMemFree(paBuf);
+    paBuf = NULL;
+
+    if (pcSamplesCaptured)
+        *pcSamplesCaptured = csCapturedTotal;
+
+    return rc;
 }
 
 static DECLCALLBACK(int) drvHostNullAudioControlIn(PPDMIHOSTAUDIO pInterface, PPDMAUDIOHSTSTRMIN pHstStrmIn,
@@ -224,13 +274,19 @@ static DECLCALLBACK(int) drvHostNullAudioControlOut(PPDMIHOSTAUDIO pInterface, P
 
 static DECLCALLBACK(int) drvHostNullAudioFiniIn(PPDMIHOSTAUDIO pInterface, PPDMAUDIOHSTSTRMIN pHstStrmIn)
 {
+    RT_NOREF(pInterface, pHstStrmIn);
     return VINF_SUCCESS;
 }
 
 static DECLCALLBACK(int) drvHostNullAudioFiniOut(PPDMIHOSTAUDIO pInterface, PPDMAUDIOHSTSTRMOUT pHstStrmOut)
 {
-    NULLAUDIOSTREAMOUT *pNullStrmOut = (NULLAUDIOSTREAMOUT *)pHstStrmOut;
-    RTMemFree(pNullStrmOut->pu8PlayBuffer);
+    RT_NOREF(pInterface);
+    PNULLAUDIOSTREAMOUT pNullStrmOut = (PNULLAUDIOSTREAMOUT)pHstStrmOut;
+    if (   pNullStrmOut
+        && pNullStrmOut->pu8PlayBuffer)
+    {
+        RTMemFree(pNullStrmOut->pu8PlayBuffer);
+    }
     return VINF_SUCCESS;
 }
 
@@ -259,8 +315,8 @@ static DECLCALLBACK(void) drvHostNullAudioShutdown(PPDMIHOSTAUDIO pInterface)
  */
 static DECLCALLBACK(int) drvHostNullAudioConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfg, uint32_t fFlags)
 {
-    AssertPtrReturn(pDrvIns, VERR_INVALID_POINTER);
-    /* pCfg is optional. */
+    RT_NOREF(pCfg, fFlags);
+    PDMDRV_CHECK_VERSIONS_RETURN(pDrvIns);
 
     PDRVHOSTNULLAUDIO pThis = PDMINS_2_DATA(pDrvIns, PDRVHOSTNULLAUDIO);
     LogRel(("Audio: Initializing NULL driver\n"));
