@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2014-2015 Oracle Corporation
+ * Copyright (C) 2014-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -65,8 +65,10 @@ typedef struct OSSAUDIOSTREAMIN
     int                hFile;
     int                cFragments;
     int                cbFragmentSize;
-    void              *pvBuf;
-    size_t             cbBuf;
+    /** Own PCM buffer. */
+    void              *pvPCMBuf;
+    /** Size (in bytes) of own PCM buffer. */
+    size_t             cbPCMBuf;
     int                old_optr;
 } OSSAUDIOSTREAMIN, *POSSAUDIOSTREAMIN;
 
@@ -78,9 +80,14 @@ typedef struct OSSAUDIOSTREAMOUT
     int                 cFragments;
     int                 cbFragmentSize;
 #ifndef RT_OS_L4
+    /** Whether we use a memory mapped file instead of our
+     *  own allocated PCM buffer below. */
     bool                fMemMapped;
 #endif
+    /** Own PCM buffer in case memory mapping is unavailable. */
     void               *pvPCMBuf;
+    /** Size (in bytes) of own PCM buffer. */
+    size_t              cbPCMBuf;
     int                 old_optr;
 } OSSAUDIOSTREAMOUT, *POSSAUDIOSTREAMOUT;
 
@@ -200,7 +207,7 @@ static int drvHostOSSAudioOSSToFmt(int fmt,
 
 static int drvHostOSSAudioClose(int *phFile)
 {
-    if (!phFile || !*phFile)
+    if (!phFile || *phFile < 1)
         return VINF_SUCCESS;
 
     int rc;
@@ -228,7 +235,7 @@ static int drvHostOSSAudioOpen(bool fIn,
     AssertPtrReturn(phFile, VERR_INVALID_POINTER);
 
     int rc;
-    int hFile;
+    int hFile = -1;
 
     do
     {
@@ -244,7 +251,8 @@ static int drvHostOSSAudioOpen(bool fIn,
         hFile = open(pszDev, (fIn ? O_RDONLY : O_WRONLY) | O_NONBLOCK);
         if (hFile == -1)
         {
-            LogRel(("OSS: Failed to open %s: %s(%d)\n", pszDev, strerror(errno), errno));
+            LogRel(("OSS: Failed to open %s for %s: %s(%d)\n",
+                    pszDev, fIn ? "input" : "output", strerror(errno), errno));
             rc = RTErrConvertFromErrno(errno);
             break;
         }
@@ -252,7 +260,7 @@ static int drvHostOSSAudioOpen(bool fIn,
         int iFormat = drvHostOSSAudioFmtToOSS(pReq->enmFormat);
         if (ioctl(hFile, SNDCTL_DSP_SAMPLESIZE, &iFormat))
         {
-            LogRel(("OSS: Failed to set audio format to %ld errno=%s(%d)\n", iFormat, strerror(errno), errno));
+            LogRel(("OSS: Failed to set audio format to %d errno=%s(%d)\n", iFormat, strerror(errno), errno));
             rc = RTErrConvertFromErrno(errno);
             break;
         }
@@ -351,9 +359,10 @@ static DECLCALLBACK(int) drvHostOSSAudioControlOut(PPDMIHOSTAUDIO pInterface, PP
     switch (enmStreamCmd)
     {
         case PDMAUDIOSTREAMCMD_ENABLE:
+        case PDMAUDIOSTREAMCMD_RESUME:
         {
-            drvAudioClearBuf(&pHstStrmOut->Props,
-                             pThisStrmOut->pvPCMBuf, AudioMixBufSize(&pHstStrmOut->MixBuf));
+            DrvAudioClearBuf(&pHstStrmOut->Props,
+                             pThisStrmOut->pvPCMBuf, pThisStrmOut->cbPCMBuf, AudioMixBufSize(&pHstStrmOut->MixBuf));
 
             mask = PCM_ENABLE_OUTPUT;
             if (ioctl(pThisStrmOut->hFile, SNDCTL_DSP_SETTRIGGER, &mask) < 0)
@@ -366,6 +375,7 @@ static DECLCALLBACK(int) drvHostOSSAudioControlOut(PPDMIHOSTAUDIO pInterface, PP
         }
 
         case PDMAUDIOSTREAMCMD_DISABLE:
+        case PDMAUDIOSTREAMCMD_PAUSE:
         {
             mask = 0;
             if (ioctl(pThisStrmOut->hFile, SNDCTL_DSP_SETTRIGGER, &mask) < 0)
@@ -393,6 +403,11 @@ static DECLCALLBACK(int) drvHostOSSAudioInit(PPDMIHOSTAUDIO pInterface)
 
     LogFlowFuncEnter();
 
+#ifdef VBOX_AUDIO_DEBUG_DUMP_PCM_DATA
+    RTFileDelete(VBOX_AUDIO_DEBUG_DUMP_PCM_DATA_PATH "ossCaptureIn.pcm");
+    RTFileDelete(VBOX_AUDIO_DEBUG_DUMP_PCM_DATA_PATH "ossPlayOut.pcm");
+#endif
+
     return VINF_SUCCESS;
 }
 
@@ -405,24 +420,22 @@ static DECLCALLBACK(int) drvHostOSSAudioCaptureIn(PPDMIHOSTAUDIO pInterface, PPD
     POSSAUDIOSTREAMIN pThisStrmIn = (POSSAUDIOSTREAMIN)pHstStrmIn;
 
     int rc = VINF_SUCCESS;
-    size_t cbToRead = RT_MIN(pThisStrmIn->cbBuf,
-                             AudioMixBufFreeBytes(&pHstStrmIn->MixBuf));
 
-    LogFlowFunc(("cbToRead=%zu\n", cbToRead));
+    AssertPtr(pHstStrmIn->pGstStrmIn);
+    size_t cbToRead = RT_MIN(pThisStrmIn->cbPCMBuf, AudioMixBufFreeBytes(&pHstStrmIn->pGstStrmIn->MixBuf));
 
-    uint32_t cWrittenTotal = 0;
-    uint32_t cbTemp;
-    ssize_t  cbRead;
+    uint32_t csCapturedTotal = 0;
     size_t   offWrite = 0;
 
+    /* Using the host side buffer as an intermediate (and not circular) buffer. */
+    AudioMixBufReset(&pHstStrmIn->MixBuf);
+
     while (cbToRead)
     {
-        cbTemp = RT_MIN(cbToRead, pThisStrmIn->cbBuf);
-        AssertBreakStmt(cbTemp, rc = VERR_NO_DATA);
-        cbRead = read(pThisStrmIn->hFile, (uint8_t *)pThisStrmIn->pvBuf + offWrite, cbTemp);
+        uint32_t cbChunk = RT_MIN(cbToRead, AudioMixBufFreeBytes(&pHstStrmIn->pGstStrmIn->MixBuf));
+        ssize_t  cbRead  = read(pThisStrmIn->hFile, (uint8_t *)pThisStrmIn->pvPCMBuf + offWrite, cbChunk);
 
-        LogFlowFunc(("cbRead=%zi, cbTemp=%RU32, cbToRead=%zu\n",
-                     cbRead, cbTemp, cbToRead));
+        LogFlowFunc(("cbRead=%zi, cbChunk=%RU32, cbToRead=%zu\n", cbRead, cbChunk, cbToRead));
 
         if (cbRead < 0)
         {
@@ -442,7 +455,7 @@ static DECLCALLBACK(int) drvHostOSSAudioCaptureIn(PPDMIHOSTAUDIO pInterface, PPD
 
                 default:
                     LogFlowFunc(("Failed to read %zu input frames, rc=%Rrc\n",
-                                 cbTemp, rc));
+                                 cbChunk, rc));
                     rc = VERR_GENERAL_FAILURE; /** @todo */
                     break;
             }
@@ -452,19 +465,33 @@ static DECLCALLBACK(int) drvHostOSSAudioCaptureIn(PPDMIHOSTAUDIO pInterface, PPD
         }
         else if (cbRead)
         {
-            uint32_t cWritten;
-            rc = AudioMixBufWriteCirc(&pHstStrmIn->MixBuf,
-                                      pThisStrmIn->pvBuf, cbRead,
-                                      &cWritten);
+#ifdef VBOX_AUDIO_DEBUG_DUMP_PCM_DATA
+            RTFILE fh;
+            RTFileOpen(&fh, VBOX_AUDIO_DEBUG_DUMP_PCM_DATA_PATH "ossCaptureIn.pcm",
+                       RTFILE_O_OPEN_CREATE | RTFILE_O_APPEND | RTFILE_O_WRITE | RTFILE_O_DENY_NONE);
+            RTFileWrite(fh, (uint8_t *)pThisStrmIn->pvPCMBuf + offWrite, cbRead, NULL);
+            RTFileClose(fh);
+#endif
+            uint32_t csWritten = 0;
+            rc = AudioMixBufWriteAt(&pHstStrmIn->MixBuf, 0 /* Offset */,
+                                    (uint8_t *)pThisStrmIn->pvPCMBuf + offWrite, cbRead,
+                                    &csWritten);
             if (RT_FAILURE(rc))
                 break;
 
-            uint32_t cbWritten = AUDIOMIXBUF_S2B(&pHstStrmIn->MixBuf, cWritten);
+            uint32_t csMixed = 0;
+            rc = AudioMixBufMixToParentEx(&pHstStrmIn->MixBuf, 0 /* Offset */, csWritten, &csMixed);
+            if (RT_FAILURE(rc))
+                break;
+
+            const uint32_t cbWritten = AUDIOMIXBUF_S2B(&pHstStrmIn->MixBuf, csWritten);
 
             Assert(cbToRead >= cbWritten);
-            cbToRead      -= cbWritten;
-            offWrite      += cbWritten;
-            cWrittenTotal += cWritten;
+            cbToRead        -= cbWritten;
+            offWrite        += cbWritten;
+            Assert(offWrite <= pThisStrmIn->cbPCMBuf);
+
+            csCapturedTotal += csMixed;
         }
         else /* No more data, try next round. */
             break;
@@ -475,16 +502,8 @@ static DECLCALLBACK(int) drvHostOSSAudioCaptureIn(PPDMIHOSTAUDIO pInterface, PPD
 
     if (RT_SUCCESS(rc))
     {
-        uint32_t cProcessed = 0;
-        if (cWrittenTotal)
-            rc = AudioMixBufMixToParent(&pHstStrmIn->MixBuf, cWrittenTotal,
-                                        &cProcessed);
-
         if (pcSamplesCaptured)
-            *pcSamplesCaptured = cWrittenTotal;
-
-        LogFlowFunc(("cWrittenTotal=%RU32 (%RU32 processed), rc=%Rrc\n",
-                     cWrittenTotal, cProcessed, rc));
+            *pcSamplesCaptured = csCapturedTotal;
     }
 
     LogFlowFuncLeaveRC(rc);
@@ -500,12 +519,16 @@ static DECLCALLBACK(int) drvHostOSSAudioFiniIn(PPDMIHOSTAUDIO pInterface, PPDMAU
 
     LogFlowFuncEnter();
 
-    if (pThisStrmIn->pvBuf)
+    if (pThisStrmIn->pvPCMBuf)
     {
-        RTMemFree(pThisStrmIn->pvBuf);
-        pThisStrmIn->pvBuf = NULL;
+        Assert(pThisStrmIn->cbPCMBuf);
+
+        RTMemFree(pThisStrmIn->pvPCMBuf);
+        pThisStrmIn->pvPCMBuf = NULL;
     }
 
+    pThisStrmIn->cbPCMBuf = 0;
+
     return VINF_SUCCESS;
 }
 
@@ -523,9 +546,13 @@ static DECLCALLBACK(int) drvHostOSSAudioFiniOut(PPDMIHOSTAUDIO pInterface, PPDMA
     {
         if (pThisStrmOut->pvPCMBuf)
         {
+            Assert(pThisStrmOut->cbPCMBuf);
+
             RTMemFree(pThisStrmOut->pvPCMBuf);
             pThisStrmOut->pvPCMBuf = NULL;
         }
+
+        pThisStrmOut->cbPCMBuf = 0;
     }
 #endif
 
@@ -545,13 +572,13 @@ static DECLCALLBACK(int) drvHostOSSAudioGetConf(PPDMIHOSTAUDIO pInterface, PPDMA
 }
 
 static DECLCALLBACK(int) drvHostOSSAudioInitIn(PPDMIHOSTAUDIO pInterface,
-                                               PPDMAUDIOHSTSTRMIN pHstStrmIn, PPDMAUDIOSTREAMCFG pCfg,
-                                               PDMAUDIORECSOURCE enmRecSource,
-                                               uint32_t *pcSamples)
+                                               PPDMAUDIOHSTSTRMIN pHstStrmIn,
+                                               PPDMAUDIOSTREAMCFG pCfgReq, PPDMAUDIOSTREAMCFG pCfgAcq,
+                                               PDMAUDIORECSOURCE enmRecSource, uint32_t *pcSamples)
 {
-    NOREF(pInterface);
+    RT_NOREF(pInterface, pCfgAcq, enmRecSource);
     AssertPtrReturn(pHstStrmIn, VERR_INVALID_POINTER);
-    AssertPtrReturn(pCfg, VERR_INVALID_POINTER);
+    AssertPtrReturn(pCfgReq,    VERR_INVALID_POINTER);
 
     POSSAUDIOSTREAMIN pThisStrmIn = (POSSAUDIOSTREAMIN)pHstStrmIn;
 
@@ -563,9 +590,9 @@ static DECLCALLBACK(int) drvHostOSSAudioInitIn(PPDMIHOSTAUDIO pInterface,
         uint32_t cSamples;
 
         OSSAUDIOSTREAMCFG reqStream, obtStream;
-        reqStream.enmFormat      = pCfg->enmFormat;
-        reqStream.uFreq          = pCfg->uHz;
-        reqStream.cChannels      = pCfg->cChannels;
+        reqStream.enmFormat      = pCfgReq->enmFormat;
+        reqStream.uFreq          = pCfgReq->uHz;
+        reqStream.cChannels      = pCfgReq->cChannels;
         reqStream.cFragments     = s_OSSConf.nfrags;
         reqStream.cbFragmentSize = s_OSSConf.fragsize;
 
@@ -583,10 +610,10 @@ static DECLCALLBACK(int) drvHostOSSAudioInitIn(PPDMIHOSTAUDIO pInterface,
             PDMAUDIOSTREAMCFG streamCfg;
             streamCfg.enmFormat     = obtStream.enmFormat;
             streamCfg.uHz           = obtStream.uFreq;
-            streamCfg.cChannels     = pCfg->cChannels;
+            streamCfg.cChannels     = pCfgReq->cChannels;
             streamCfg.enmEndianness = obtStream.enmENDIANNESS;
 
-            rc = drvAudioStreamCfgToProps(&streamCfg, &pHstStrmIn->Props);
+            rc = DrvAudioStreamCfgToProps(&streamCfg, &pHstStrmIn->Props);
             if (RT_SUCCESS(rc))
             {
                 cSamples = (obtStream.cFragments * obtStream.cbFragmentSize)
@@ -598,16 +625,16 @@ static DECLCALLBACK(int) drvHostOSSAudioInitIn(PPDMIHOSTAUDIO pInterface,
 
         if (RT_SUCCESS(rc))
         {
-            size_t cbBuf = cSamples * (1 << pHstStrmIn->Props.cShift);
-            pThisStrmIn->pvBuf = RTMemAlloc(cbBuf);
-            if (!pThisStrmIn->pvBuf)
+            size_t cbSample = (1 << pHstStrmIn->Props.cShift);
+            size_t cbBuf    = cSamples * cbSample;
+            pThisStrmIn->pvPCMBuf = RTMemAlloc(cbBuf);
+            if (!pThisStrmIn->pvPCMBuf)
             {
-                LogRel(("OSS: Failed allocating ADC buffer with %RU32 samples, each %d bytes\n",
-                        cSamples, 1 << pHstStrmIn->Props.cShift));
+                LogRel(("OSS: Failed allocating ADC buffer with %RU32 samples (%zu bytes per sample)\n", cSamples, cbSample));
                 rc = VERR_NO_MEMORY;
             }
 
-            pThisStrmIn->cbBuf = cbBuf;
+            pThisStrmIn->cbPCMBuf = cbBuf;
 
             if (pcSamples)
                 *pcSamples = cSamples;
@@ -623,12 +650,13 @@ static DECLCALLBACK(int) drvHostOSSAudioInitIn(PPDMIHOSTAUDIO pInterface,
 }
 
 static DECLCALLBACK(int) drvHostOSSAudioInitOut(PPDMIHOSTAUDIO pInterface,
-                                                PPDMAUDIOHSTSTRMOUT pHstStrmOut, PPDMAUDIOSTREAMCFG pCfg,
+                                                PPDMAUDIOHSTSTRMOUT pHstStrmOut,
+                                                PPDMAUDIOSTREAMCFG pCfgReq, PPDMAUDIOSTREAMCFG pCfgAcq,
                                                 uint32_t *pcSamples)
 {
-    NOREF(pInterface);
+    RT_NOREF(pInterface, pCfgAcq);
     AssertPtrReturn(pHstStrmOut, VERR_INVALID_POINTER);
-    AssertPtrReturn(pCfg, VERR_INVALID_POINTER);
+    AssertPtrReturn(pCfgReq,     VERR_INVALID_POINTER);
 
     POSSAUDIOSTREAMOUT pThisStrmOut = (POSSAUDIOSTREAMOUT)pHstStrmOut;
 
@@ -640,9 +668,9 @@ static DECLCALLBACK(int) drvHostOSSAudioInitOut(PPDMIHOSTAUDIO pInterface,
         uint32_t cSamples;
 
         OSSAUDIOSTREAMCFG reqStream, obtStream;
-        reqStream.enmFormat      = pCfg->enmFormat;
-        reqStream.uFreq          = pCfg->uHz;
-        reqStream.cChannels      = pCfg->cChannels;
+        reqStream.enmFormat      = pCfgReq->enmFormat;
+        reqStream.uFreq          = pCfgReq->uHz;
+        reqStream.cChannels      = pCfgReq->cChannels;
         reqStream.cFragments     = s_OSSConf.nfrags;
         reqStream.cbFragmentSize = s_OSSConf.fragsize;
 
@@ -660,10 +688,10 @@ static DECLCALLBACK(int) drvHostOSSAudioInitOut(PPDMIHOSTAUDIO pInterface,
             PDMAUDIOSTREAMCFG streamCfg;
             streamCfg.enmFormat     = obtStream.enmFormat;
             streamCfg.uHz           = obtStream.uFreq;
-            streamCfg.cChannels     = pCfg->cChannels;
+            streamCfg.cChannels     = pCfgReq->cChannels;
             streamCfg.enmEndianness = obtStream.enmENDIANNESS;
 
-            rc = drvAudioStreamCfgToProps(&streamCfg, &pHstStrmOut->Props);
+            rc = DrvAudioStreamCfgToProps(&streamCfg, &pHstStrmOut->Props);
             if (RT_SUCCESS(rc))
                 cSamples = (obtStream.cFragments * obtStream.cbFragmentSize)
                            >> pHstStrmOut->Props.cShift;
@@ -713,9 +741,7 @@ static DECLCALLBACK(int) drvHostOSSAudioInitOut(PPDMIHOSTAUDIO pInterface,
                         int rc2 = munmap(pThisStrmOut->pvPCMBuf,
                                          cSamples << pHstStrmOut->Props.cShift);
                         if (rc2)
-                            LogRel(("OSS: Failed to unmap DAC output file: %s\n",
-                                    strerror(errno)));
-
+                            LogRel(("OSS: Failed to unmap DAC output file: %s\n", strerror(errno)));
                         break;
                     }
                 }
@@ -727,15 +753,20 @@ static DECLCALLBACK(int) drvHostOSSAudioInitOut(PPDMIHOSTAUDIO pInterface,
             if (!pThisStrmOut->fMemMapped)
             {
 #endif
+                size_t cbSample = (1 << pHstStrmOut->Props.cShift);
+                size_t cbPCMBuf = cSamples * cbSample;
+
                 LogFlowFunc(("cSamples=%RU32\n", cSamples));
-                pThisStrmOut->pvPCMBuf = RTMemAlloc(cSamples * (1 << pHstStrmOut->Props.cShift));
+
+                pThisStrmOut->pvPCMBuf = RTMemAlloc(cbPCMBuf);
                 if (!pThisStrmOut->pvPCMBuf)
                 {
-                    LogRel(("OSS: Failed allocating DAC buffer with %RU32 samples, each %d bytes\n",
-                            cSamples, 1 << pHstStrmOut->Props.cShift));
+                    LogRel(("OSS: Failed allocating DAC buffer with %RU32 samples (%zu bytes per sample)\n", cSamples, cbSample));
                     rc = VERR_NO_MEMORY;
                     break;
                 }
+
+                pThisStrmOut->cbPCMBuf = cbPCMBuf;
 #ifndef RT_OS_L4
             }
 #endif
@@ -775,7 +806,7 @@ static DECLCALLBACK(int) drvHostOSSAudioPlayOut(PPDMIHOSTAUDIO pInterface, PPDMA
     {
         size_t cbBuf = AudioMixBufSizeBytes(&pHstStrmOut->MixBuf);
 
-        uint32_t cLive = drvAudioHstOutSamplesLive(pHstStrmOut);
+        uint32_t cLive = AudioMixBufLive(&pHstStrmOut->MixBuf);
         uint32_t cToRead;
 
 #ifndef RT_OS_L4
@@ -857,11 +888,24 @@ static DECLCALLBACK(int) drvHostOSSAudioPlayOut(PPDMIHOSTAUDIO pInterface, PPDMA
                                       cbRead);
             if (cbWritten == -1)
             {
-                LogRel(("OSS: Failed writing output data %s\n", strerror(errno)));
                 rc = RTErrConvertFromErrno(errno);
+                if (rc == VERR_TRY_AGAIN) /* OSS is busy -- do not report this back as error. */
+                {
+                    rc = VINF_SUCCESS;
+                    break;
+                }
+
+                LogRel(("OSS: Failed writing output data: %s\n", strerror(errno)));
                 break;
             }
 
+#ifdef VBOX_AUDIO_DEBUG_DUMP_PCM_DATA
+            RTFILE fh;
+            RTFileOpen(&fh, VBOX_AUDIO_DEBUG_DUMP_PCM_DATA_PATH "ossPlayOut.pcm",
+                       RTFILE_O_OPEN_CREATE | RTFILE_O_APPEND | RTFILE_O_WRITE | RTFILE_O_DENY_NONE);
+            RTFileWrite(fh, pThisStrmOut->pvPCMBuf, cbRead, NULL);
+            RTFileClose(fh);
+#endif
             Assert(cbToRead >= cbRead);
             cbToRead -= cbRead;
             cbReadTotal += cbRead;
@@ -917,6 +961,7 @@ static DECLCALLBACK(void *) drvHostOSSAudioQueryInterface(PPDMIBASE pInterface,
  */
 static DECLCALLBACK(int) drvHostOSSAudioConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfg, uint32_t fFlags)
 {
+    RT_NOREF(pCfg, fFlags);
     PDRVHOSTOSSAUDIO pThis = PDMINS_2_DATA(pDrvIns, PDRVHOSTOSSAUDIO);
     LogRel(("Audio: Initializing OSS driver\n"));
 
