@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -52,9 +52,9 @@
 
 #define UNSUPPORTED_FUNCTION    0x86    /* Specific to INT 15h. */
 
-#define BIOS_CONFIG_TABLE       0xe6f5  /* TODO: configurable? put elsewhere? */
+#define BIOS_CONFIG_TABLE       0xe6f5  /** @todo configurable? put elsewhere? */
 
-#define ACPI_DATA_SIZE    0x00010000L   /* TODO: configurable? put elsewhere? */
+#define ACPI_DATA_SIZE    0x00010000L   /** @todo configurable? put elsewhere? */
 
 #define BX_CPU                  3
 
@@ -64,7 +64,13 @@ extern  int rmode_IDT;
 uint16_t read_ss(void);
 #pragma aux read_ss = "mov ax, ss" modify exact [ax] nomemory;
 
-void pm_stack_save(uint16_t cx, uint16_t es, uint16_t si);
+#if VBOX_BIOS_CPU >= 80386
+
+/* The 386+ code uses CR0 to switch to/from protected mode.
+ * Quite straightforward.
+ */
+
+void pm_stack_save(uint16_t cx, uint16_t es, uint16_t si, uint16_t frame);
 #pragma aux pm_stack_save =     \
     ".386"                      \
     "push   ds"                 \
@@ -73,7 +79,7 @@ void pm_stack_save(uint16_t cx, uint16_t es, uint16_t si);
     "mov    ds, ax"             \
     "mov    ds:[467h], sp"      \
     "mov    ds:[469h], ss"      \
-    parm [cx] [es] [si] modify nomemory;
+    parm [cx] [es] [si] [ax] modify nomemory;
 
 /* Uses position independent code... because it was too hard to figure
  * out how to code the far call in inline assembler.
@@ -102,14 +108,6 @@ void pm_enter(void);
     "mov    es, ax"                 \
     modify nomemory;
 
-void pm_copy(void);
-#pragma aux pm_copy =               \
-    "xor    si, si"                 \
-    "xor    di, di"                 \
-    "cld"                           \
-    "rep    movsw"                  \
-    modify nomemory;
-
 /* Restore segment limits to real mode compatible values and
  * return to real mode.
  */
@@ -147,6 +145,81 @@ void pm_stack_restore(void);
     "pop    ds"                 \
     modify nomemory;
 
+#elif VBOX_BIOS_CPU >= 80286
+
+/* The 286 code uses LMSW to switch to protected mode but it has to reset
+ * the CPU to get back to real mode. Ugly! See return_blkmove in orgs.asm
+ * for the other matching half.
+ */
+void pm_stack_save(uint16_t cx, uint16_t es, uint16_t si, uint16_t frame);
+#pragma aux pm_stack_save =     \
+    "xor    ax, ax"             \
+    "mov    ds, ax"             \
+    "mov    ds:[467h], bx"      \
+    "mov    ds:[469h], ss"      \
+    parm [cx] [es] [si] [bx] modify nomemory;
+
+/* Uses position independent code... because it was too hard to figure
+ * out how to code the far call in inline assembler.
+ * NB: Trashes MSW bits but the CPU will be reset anyway.
+ */
+void pm_enter(void);
+#pragma aux pm_enter =              \
+    ".286p"                         \
+    "call   pentry"                 \
+    "pentry:"                       \
+    "pop    di"                     \
+    "add    di, 18h"                \
+    "push   20h"                    \
+    "push   di"                     \
+    "lgdt   fword ptr es:[si+8]"    \
+    "lidt   fword ptr cs:pmode_IDT" \
+    "or     al, 1"                  \
+    "lmsw   ax"                     \
+    "retf"                          \
+    "pm_pm:"                        \
+    "mov    ax, 28h"                \
+    "mov    ss, ax"                 \
+    "mov    ax, 10h"                \
+    "mov    ds, ax"                 \
+    "mov    ax, 18h"                \
+    "mov    es, ax"                 \
+    modify nomemory;
+
+/* Set up shutdown status and reset the CPU. The POST code
+ * will regain control. Port 80h is written with status.
+ * Code 9 is written to CMOS shutdown status byte (0Fh).
+ * CPU is triple faulted.                                                    .
+ */
+void pm_exit(void);
+#pragma aux pm_exit =               \
+    "xor    ax, ax"                 \
+    "out    80h, al"                \
+    "mov    al, 0Fh"                \
+    "out    70h, al"                \
+    "mov    al, 09h"                \
+    "out    71h, al"                \
+    ".286p"                         \
+    "lidt   fword ptr cs:pmode_IDT" \
+    "int    3"                      \
+    modify nomemory;
+
+/* Dummy. Actually done in return_blkmove. */
+void pm_stack_restore(void);
+#pragma aux pm_stack_restore =  \
+    "rm_return:"                \
+    modify nomemory;
+
+#endif
+
+void pm_copy(void);
+#pragma aux pm_copy =               \
+    "xor    si, si"                 \
+    "xor    di, di"                 \
+    "cld"                           \
+    "rep    movsw"                  \
+    modify nomemory;
+
 /* The pm_switch has a few crucial differences from pm_enter, hence
  * it is replicated here. Uses LMSW to avoid trashing high word of eax.
  */
@@ -191,50 +264,43 @@ void pm_unwind(uint16_t args);
     "push   cx"             \
     "retf"                  \
     parm [ax] modify nomemory aborts;
-    
-// @todo: This method is silly. The RTC should be programmed to fire an interrupt
-// instead of hogging the CPU with inaccurate code.
-void timer_wait(uint16_t lo, uint16_t hi);
-#pragma aux timer_wait =    \
-    ".386"                  \
-    "shl    eax, 16"        \
-    "mov    ax, dx"         \
-    "mov    ebx, 15"        \
-    "xor    edx, edx"       \
-    "div    ebx"            \
-    "mov    ecx, eax"       \
-    "in     al, 61h"        \
-    "and    al, 10h"        \
-    "mov    ah, al"         \
-    "or     ecx, ecx"       \
-    "je int1586_tick_end"   \
-    "int1586_tick:"         \
-    "in     al, 61h"        \
-    "and    al, 10h"        \
-    "cmp    al, ah"         \
-    "je     int1586_tick"   \
-    "mov    ah, al"         \
-    "dec    ecx"            \
-    "jnz    int1586_tick"   \
-    "int1586_tick_end:"     \
-    parm [dx] [ax] modify [bx cx] nomemory;
 
+/// @todo This method is silly. The RTC should be programmed to fire an interrupt
+// instead of hogging the CPU with inaccurate code.
+void timer_wait(uint32_t usec_wait)
+{
+    uint32_t    cycles;
+    uint8_t     old_val;
+    uint8_t     cur_val;
+
+    /* We wait in 15 usec increments. */
+    cycles = usec_wait / 15;
+
+    old_val = inp(0x61) & 0x10;
+    while (cycles--) {
+        /* Wait 15us. */
+        do {
+            cur_val = inp(0x61) & 0x10;
+        } while (cur_val != old_val);
+        old_val = cur_val;
+    }
+}
 
 bx_bool set_enable_a20(bx_bool val)
 {
     uint8_t     oldval;
-    
+
     // Use PS/2 System Control port A to set A20 enable
-    
+
     // get current setting first
     oldval = inb(0x92);
-    
+
     // change A20 status
     if (val)
         outb(0x92, oldval | 0x02);
     else
         outb(0x92, oldval & 0xfd);
-    
+
     return((oldval & 0x02) != 0);
 }
 
@@ -261,7 +327,7 @@ void set_e820_range(uint16_t ES, uint16_t DI, uint32_t start, uint32_t end,
     range->type   = type;
 }
 
-// @todo: move elsewhere?
+/// @todo move elsewhere?
 #define AX      r.gr.u.r16.ax
 #define BX      r.gr.u.r16.bx
 #define CX      r.gr.u.r16.cx
@@ -275,15 +341,13 @@ void set_e820_range(uint16_t ES, uint16_t DI, uint32_t start, uint32_t end,
 #define EBX     r.gr.u.r32.ebx
 #define ECX     r.gr.u.r32.ecx
 #define EDX     r.gr.u.r32.edx
+#define ESI     r.gr.u.r32.esi
+#define EDI     r.gr.u.r32.edi
 #define ES      r.es
 
 
 void BIOSCALL int15_function(sys_regs_t r)
 {
-    bx_bool     prev_a20_enable;
-    uint16_t    base15_00;
-    uint8_t     base23_16;
-    uint16_t    ss;
     uint16_t    bRegister;
     uint8_t     irqDisable;
 
@@ -335,7 +399,7 @@ void BIOSCALL int15_function(sys_regs_t r)
         SET_AH(UNSUPPORTED_FUNCTION);
         break;
 
-    //@todo: Why does this need special handling? All we need is to set CF
+    /// @todo Why does this need special handling? All we need is to set CF
     //       but not handle this as an unknown function (regardless of CPU type).
     case 0x4f:
         /* keyboard intercept */
@@ -353,7 +417,7 @@ void BIOSCALL int15_function(sys_regs_t r)
         break;
 
     case 0x83: {
-        if( GET_AL() ) {
+        if( GET_AL() == 0 ) {
             // Set Interval requested.
             if( ( read_byte( 0x40, 0xA0 ) & 1 ) == 0 ) {
                 // Interval not already set.
@@ -385,86 +449,10 @@ void BIOSCALL int15_function(sys_regs_t r)
             SET_AH(UNSUPPORTED_FUNCTION);
             SET_AL(GET_AL() - 1);
         }
-        
+
         break;
         }
 
-    case 0x87:
-#if BX_CPU < 3
-        SET_AH(UNSUPPORTED_FUNCTION);
-        SET_CF();
-#endif
-        // +++ should probably have descriptor checks
-        // +++ should have exception handlers
-        
-        // turn off interrupts
-        int_disable();    //@todo: aren't they disabled already?
-        
-        prev_a20_enable = set_enable_a20(1); // enable A20 line
-        
-        // 128K max of transfer on 386+ ???
-        // source == destination ???
-        
-        // ES:SI points to descriptor table
-        // offset   use     initially  comments
-        // ==============================================
-        // 00..07   Unused  zeros      Null descriptor
-        // 08..0f   GDT     zeros      filled in by BIOS
-        // 10..17   source  ssssssss   source of data
-        // 18..1f   dest    dddddddd   destination of data
-        // 20..27   CS      zeros      filled in by BIOS
-        // 28..2f   SS      zeros      filled in by BIOS
-        
-        //es:si
-        //eeee0
-        //0ssss
-        //-----
-        
-        // check for access rights of source & dest here
-        
-        // Initialize GDT descriptor
-        base15_00 = (ES << 4) + SI;
-        base23_16 = ES >> 12;
-        if (base15_00 < (ES<<4))
-            base23_16++;
-        write_word(ES, SI+0x08+0, 47);       // limit 15:00 = 6 * 8bytes/descriptor
-        write_word(ES, SI+0x08+2, base15_00);// base 15:00
-        write_byte(ES, SI+0x08+4, base23_16);// base 23:16
-        write_byte(ES, SI+0x08+5, 0x93);     // access
-        write_word(ES, SI+0x08+6, 0x0000);   // base 31:24/reserved/limit 19:16
-        
-        // Initialize CS descriptor
-        write_word(ES, SI+0x20+0, 0xffff);// limit 15:00 = normal 64K limit
-        write_word(ES, SI+0x20+2, 0x0000);// base 15:00
-        write_byte(ES, SI+0x20+4, 0x000f);// base 23:16
-        write_byte(ES, SI+0x20+5, 0x9b);  // access
-        write_word(ES, SI+0x20+6, 0x0000);// base 31:24/reserved/limit 19:16
-        
-        // Initialize SS descriptor
-        ss = read_ss();
-        base15_00 = ss << 4;
-        base23_16 = ss >> 12;
-        write_word(ES, SI+0x28+0, 0xffff);   // limit 15:00 = normal 64K limit
-        write_word(ES, SI+0x28+2, base15_00);// base 15:00
-        write_byte(ES, SI+0x28+4, base23_16);// base 23:16
-        write_byte(ES, SI+0x28+5, 0x93);     // access
-        write_word(ES, SI+0x28+6, 0x0000);   // base 31:24/reserved/limit 19:16
-        
-        pm_stack_save(CX, ES, SI);
-        pm_enter();
-        pm_copy();
-        pm_exit();
-        pm_stack_restore();
-        
-        set_enable_a20(prev_a20_enable);
-        
-        // turn interrupts back on
-        int_enable();
-        
-        SET_AH(0);
-        CLEAR_CF();
-        break;
-
     case 0x88:
         // Get the amount of extended memory (above 1M)
 #if BX_CPU < 2
@@ -472,12 +460,12 @@ void BIOSCALL int15_function(sys_regs_t r)
         SET_CF();
 #else
         AX = (inb_cmos(0x31) << 8) | inb_cmos(0x30);
-        
+
         // According to Ralf Brown's interrupt the limit should be 15M,
         // but real machines mostly return max. 63M.
         if(AX > 0xffc0)
             AX = 0xffc0;
-        
+
         CLEAR_CF();
 #endif
         break;
@@ -487,12 +475,12 @@ void BIOSCALL int15_function(sys_regs_t r)
         // ES:DI points to user-supplied GDT
         // BH/BL contains starting interrupt numbers for PIC0/PIC1
         // This subfunction does not return!
-        
+
         // turn off interrupts
-        int_disable();  //@todo: aren't they off already?
-        
+        int_disable();  /// @todo aren't they off already?
+
         set_enable_a20(1); // enable A20 line; we're supposed to fail if that fails
-        
+
         // Initialize CS descriptor for BIOS
         write_word(ES, SI+0x38+0, 0xffff);// limit 15:00 = normal 64K limit
         write_word(ES, SI+0x38+2, 0x0000);// base 15:00
@@ -589,9 +577,23 @@ void BIOSCALL int15_function32(sys32_regs_t r)
         // Wait for CX:DX microseconds. currently using the
         // refresh request port 0x61 bit4, toggling every 15usec
         int_enable();
-        timer_wait(DX, CX);
+        timer_wait(((uint32_t)CX << 16) | DX);
         break;
-    
+
+    case 0xd0:
+        if (GET_AL() != 0x4f)
+            goto int15_unimplemented;
+        if (EBX == 0x50524f43 && ECX == 0x4d4f4445 && ESI == 0 && EDI == 0)
+        {
+            CLEAR_CF();
+            ESI = EBX;
+            EDI = ECX;
+            EAX = 0x49413332;
+        }
+        else
+            goto int15_unimplemented;
+        break;
+
     case 0xe8:
         switch(GET_AL()) {
         case 0x20: // coded by osmaker aka K.J.
@@ -698,6 +700,18 @@ void BIOSCALL int15_function32(sys32_regs_t r)
                         EBX = 5;
                         break;
                     case 5:
+                        set_e820_range(ES, DI,
+                                       0xfec00000,
+                                       0xfec00000 + 0x1000, 0, 0, 2); // I/O APIC
+                        EBX = 6;
+                        break;
+                    case 6:
+                        set_e820_range(ES, DI,
+                                       0xfee00000,
+                                       0xfee00000 + 0x1000, 0, 0, 2); // Local APIC
+                        EBX = 7;
+                        break;
+                    case 7:
                         /* 256KB BIOS area at the end of 4 GB */
 #ifdef VBOX
                         /* We don't set the end to 1GB here and rely on the 32-bit
@@ -706,35 +720,37 @@ void BIOSCALL int15_function32(sys32_regs_t r)
                         set_e820_range(ES, DI,
                                        0xfffc0000L, 0x00000000L, 0, 0, 2);
                         if (mcfgStart != 0)
-                            EBX = 6;
+                            EBX = 8;
                         else
                         {
                             if (extra_highbits_memory_size || extra_lowbits_memory_size)
-                                EBX = 7;
+                                EBX = 9;
                             else
                                 EBX = 0;
                         }
                         break;
-                     case 6:
+                     case 8:
                         /* PCI MMIO config space (MCFG) */
                         set_e820_range(ES, DI,
                                        mcfgStart, mcfgStart + mcfgSize, 0, 0, 2);
 
                         if (extra_highbits_memory_size || extra_lowbits_memory_size)
-                            EBX = 7;
+                            EBX = 9;
                         else
                             EBX = 0;
                         break;
-                    case 7:
+                    case 9:
 #ifdef VBOX /* Don't succeeded if no memory above 4 GB.  */
                         /* Mapping of memory above 4 GB if present.
-                           Note: set_e820_range needs do no borrowing in the
-                                 subtraction because of the nice numbers. */
+                           Note1: set_e820_range needs do no borrowing in the
+                                  subtraction because of the nice numbers.
+                           Note2* works only up to 1TB because of uint8_t for
+                                  the upper bits!*/
                         if (extra_highbits_memory_size || extra_lowbits_memory_size)
                         {
                             set_e820_range(ES, DI,
                                            0x00000000L, extra_lowbits_memory_size,
-                                           1 /*GB*/, extra_highbits_memory_size + 1 /*GB*/, 1);
+                                           1 /*x4GB*/, extra_highbits_memory_size + 1 /*x4GB*/, 1);
                             EBX = 0;
                         }
                         break;
@@ -763,24 +779,24 @@ void BIOSCALL int15_function32(sys32_regs_t r)
         case 0x01:
             // do we have any reason to fail here ?
             CLEAR_CF();
-            
+
             // my real system sets ax and bx to 0
             // this is confirmed by Ralph Brown list
             // but syslinux v1.48 is known to behave
             // strangely if ax is set to 0
             // regs.u.r16.ax = 0;
             // regs.u.r16.bx = 0;
-            
+
             // Get the amount of extended memory (above 1M)
             CX = (inb_cmos(0x31) << 8) | inb_cmos(0x30);
-            
+
             // limit to 15M
             if(CX > 0x3c00)
                 CX = 0x3c00;
-            
+
             // Get the amount of extended memory above 16M in 64k blocks
             DX = (inb_cmos(0x35) << 8) | inb_cmos(0x34);
-            
+
             // Set configured memory equal to extended memory
             AX = CX;
             BX = DX;
@@ -799,3 +815,88 @@ void BIOSCALL int15_function32(sys32_regs_t r)
         break;
     }
 }
+
+#if VBOX_BIOS_CPU >= 80286
+
+#undef  FLAGS
+#define FLAGS   r.ra.flags.u.r16.flags
+
+/* Function 0x87 handled separately due to specific stack layout requirements. */
+void BIOSCALL int15_blkmove(disk_regs_t r)
+{
+    bx_bool     prev_a20_enable;
+    uint16_t    base15_00;
+    uint8_t     base23_16;
+    uint16_t    ss;
+
+    // +++ should probably have descriptor checks
+    // +++ should have exception handlers
+
+    // turn off interrupts
+    int_disable();    /// @todo aren't they disabled already?
+
+    prev_a20_enable = set_enable_a20(1); // enable A20 line
+
+    // 128K max of transfer on 386+ ???
+    // source == destination ???
+
+    // ES:SI points to descriptor table
+    // offset   use     initially  comments
+    // ==============================================
+    // 00..07   Unused  zeros      Null descriptor
+    // 08..0f   GDT     zeros      filled in by BIOS
+    // 10..17   source  ssssssss   source of data
+    // 18..1f   dest    dddddddd   destination of data
+    // 20..27   CS      zeros      filled in by BIOS
+    // 28..2f   SS      zeros      filled in by BIOS
+
+    //es:si
+    //eeee0
+    //0ssss
+    //-----
+
+    // check for access rights of source & dest here
+
+    // Initialize GDT descriptor
+    base15_00 = (ES << 4) + SI;
+    base23_16 = ES >> 12;
+    if (base15_00 < (ES<<4))
+        base23_16++;
+    write_word(ES, SI+0x08+0, 47);       // limit 15:00 = 6 * 8bytes/descriptor
+    write_word(ES, SI+0x08+2, base15_00);// base 15:00
+    write_byte(ES, SI+0x08+4, base23_16);// base 23:16
+    write_byte(ES, SI+0x08+5, 0x93);     // access
+    write_word(ES, SI+0x08+6, 0x0000);   // base 31:24/reserved/limit 19:16
+
+    // Initialize CS descriptor
+    write_word(ES, SI+0x20+0, 0xffff);// limit 15:00 = normal 64K limit
+    write_word(ES, SI+0x20+2, 0x0000);// base 15:00
+    write_byte(ES, SI+0x20+4, 0x000f);// base 23:16
+    write_byte(ES, SI+0x20+5, 0x9b);  // access
+    write_word(ES, SI+0x20+6, 0x0000);// base 31:24/reserved/limit 19:16
+
+    // Initialize SS descriptor
+    ss = read_ss();
+    base15_00 = ss << 4;
+    base23_16 = ss >> 12;
+    write_word(ES, SI+0x28+0, 0xffff);   // limit 15:00 = normal 64K limit
+    write_word(ES, SI+0x28+2, base15_00);// base 15:00
+    write_byte(ES, SI+0x28+4, base23_16);// base 23:16
+    write_byte(ES, SI+0x28+5, 0x93);     // access
+    write_word(ES, SI+0x28+6, 0x0000);   // base 31:24/reserved/limit 19:16
+
+    pm_stack_save(CX, ES, SI, FP_OFF(&r));
+    pm_enter();
+    pm_copy();
+    pm_exit();
+    pm_stack_restore();
+
+    set_enable_a20(prev_a20_enable);
+
+    // turn interrupts back on
+    int_enable();
+
+    SET_AH(0);
+    CLEAR_CF();
+}
+#endif
