@@ -1,4 +1,4 @@
-/* $Revision: 91907 $ */
+/* $Id: tstCAPIGlue.c $ */
 /** @file tstCAPIGlue.c
  * Demonstrator program to illustrate use of C bindings of Main API.
  *
@@ -8,7 +8,7 @@
  */
 
 /*
- * Copyright (C) 2009-2014 Oracle Corporation
+ * Copyright (C) 2009-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -19,9 +19,17 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+
+/** @todo
+ * Our appologies for the 256+ missing return code checks in this sample file.
+ *
+ * We strongly recomment users of the VBoxCAPI to check all return codes!
+ */
+
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include "VBoxCAPIGlue.h"
 #include <stdio.h>
 #include <string.h>
@@ -31,6 +39,9 @@
 # include <unistd.h>
 # include <sys/poll.h>
 #endif
+#ifdef ___iprt_cdefs_h
+# error "not supposed to involve any IPRT or VBox headers here."
+#endif
 
 /**
  * Select between active event listener (defined) and passive event listener
@@ -40,9 +51,9 @@
 #undef USE_ACTIVE_EVENT_LISTENER
 
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 /** Set by Ctrl+C handler. */
 static volatile int g_fStop = 0;
 
@@ -430,9 +441,8 @@ static IEventListenerDemoVtblInt g_IEventListenerDemoVtblInt =
  *
  * @param   virtualBox ptr to IVirtualBox object
  * @param   session    ptr to ISession object
- * @param   id         identifies the machine to start
  */
-static void registerActiveEventListener(IVirtualBox *virtualBox, ISession *session, BSTR machineId)
+static void registerActiveEventListener(IVirtualBox *virtualBox, ISession *session)
 {
     IConsole *console = NULL;
     HRESULT rc;
@@ -444,33 +454,36 @@ static void registerActiveEventListener(IVirtualBox *virtualBox, ISession *sessi
         rc = IConsole_get_EventSource(console, &es);
         if (SUCCEEDED(rc) && es)
         {
-            static const ULONG interestingEvents[] =
-                {
-                    VBoxEventType_OnMousePointerShapeChanged,
-                    VBoxEventType_OnMouseCapabilityChanged,
-                    VBoxEventType_OnKeyboardLedsChanged,
-                    VBoxEventType_OnStateChanged,
-                    VBoxEventType_OnAdditionsStateChanged,
-                    VBoxEventType_OnNetworkAdapterChanged,
-                    VBoxEventType_OnSerialPortChanged,
-                    VBoxEventType_OnParallelPortChanged,
-                    VBoxEventType_OnStorageControllerChanged,
-                    VBoxEventType_OnMediumChanged,
-                    VBoxEventType_OnVRDEServerChanged,
-                    VBoxEventType_OnUSBControllerChanged,
-                    VBoxEventType_OnUSBDeviceStateChanged,
-                    VBoxEventType_OnSharedFolderChanged,
-                    VBoxEventType_OnRuntimeError,
-                    VBoxEventType_OnCanShowWindow,
-                    VBoxEventType_OnShowWindow
-                };
+            static const ULONG s_auInterestingEvents[] =
+            {
+                VBoxEventType_OnMousePointerShapeChanged,
+                VBoxEventType_OnMouseCapabilityChanged,
+                VBoxEventType_OnKeyboardLedsChanged,
+                VBoxEventType_OnStateChanged,
+                VBoxEventType_OnAdditionsStateChanged,
+                VBoxEventType_OnNetworkAdapterChanged,
+                VBoxEventType_OnSerialPortChanged,
+                VBoxEventType_OnParallelPortChanged,
+                VBoxEventType_OnStorageControllerChanged,
+                VBoxEventType_OnMediumChanged,
+                VBoxEventType_OnVRDEServerChanged,
+                VBoxEventType_OnUSBControllerChanged,
+                VBoxEventType_OnUSBDeviceStateChanged,
+                VBoxEventType_OnSharedFolderChanged,
+                VBoxEventType_OnRuntimeError,
+                VBoxEventType_OnCanShowWindow,
+                VBoxEventType_OnShowWindow
+            };
             SAFEARRAY *interestingEventsSA = NULL;
             IEventListenerDemo *consoleListener = NULL;
 
             /* The VirtualBox API expects enum values as VT_I4, which in the
              * future can be hopefully relaxed. */
-            interestingEventsSA = g_pVBoxFuncs->pfnSafeArrayCreateVector(VT_I4, 0, sizeof(interestingEvents) / sizeof(interestingEvents[0]));
-            g_pVBoxFuncs->pfnSafeArrayCopyInParamHelper(interestingEventsSA, &interestingEvents, sizeof(interestingEvents));
+            interestingEventsSA = g_pVBoxFuncs->pfnSafeArrayCreateVector(VT_I4, 0,
+                                                                           sizeof(s_auInterestingEvents)
+                                                                         / sizeof(s_auInterestingEvents[0]));
+            g_pVBoxFuncs->pfnSafeArrayCopyInParamHelper(interestingEventsSA, &s_auInterestingEvents,
+                                                        sizeof(s_auInterestingEvents));
 
             consoleListener = calloc(1, sizeof(IEventListenerDemo));
             if (consoleListener)
@@ -497,9 +510,7 @@ static void registerActiveEventListener(IVirtualBox *virtualBox, ISession *sessi
 #endif
 
                     while (!g_fStop)
-                    {
                         g_pVBoxFuncs->pfnProcessEventQueue(250);
-                    }
 
 #ifdef WIN32
                     SetConsoleCtrlHandler(ctrlCHandler, FALSE);
@@ -508,9 +519,7 @@ static void registerActiveEventListener(IVirtualBox *virtualBox, ISession *sessi
 #endif
                 }
                 else
-                {
                     printf("Failed to register event listener.\n");
-                }
                 IEventSource_UnregisterListener(es, (IEventListener *)consoleListener);
 #ifdef WIN32
                 if (consoleListener->pUnkMarshaler)
@@ -519,16 +528,12 @@ static void registerActiveEventListener(IVirtualBox *virtualBox, ISession *sessi
                 IEventListenerDemo_Release(consoleListener);
             }
             else
-            {
                 printf("Failed while allocating memory for console event listener.\n");
-            }
             g_pVBoxFuncs->pfnSafeArrayDestroy(interestingEventsSA);
             IEventSource_Release(es);
         }
         else
-        {
             printf("Failed to get the event source instance.\n");
-        }
         IConsole_Release(console);
     }
 }
@@ -540,47 +545,49 @@ static void registerActiveEventListener(IVirtualBox *virtualBox, ISession *sessi
  *
  * @param   virtualBox ptr to IVirtualBox object
  * @param   session    ptr to ISession object
- * @param   id         identifies the machine to start
  */
-static void registerPassiveEventListener(IVirtualBox *virtualBox, ISession *session, BSTR machineId)
+static void registerPassiveEventListener(ISession *session)
 {
     IConsole *console = NULL;
     HRESULT rc;
 
     rc = ISession_get_Console(session, &console);
-    if ((SUCCEEDED(rc)) && console)
+    if (SUCCEEDED(rc) && console)
     {
         IEventSource *es = NULL;
         rc = IConsole_get_EventSource(console, &es);
         if (SUCCEEDED(rc) && es)
         {
-            static const ULONG interestingEvents[] =
-                {
-                    VBoxEventType_OnMousePointerShapeChanged,
-                    VBoxEventType_OnMouseCapabilityChanged,
-                    VBoxEventType_OnKeyboardLedsChanged,
-                    VBoxEventType_OnStateChanged,
-                    VBoxEventType_OnAdditionsStateChanged,
-                    VBoxEventType_OnNetworkAdapterChanged,
-                    VBoxEventType_OnSerialPortChanged,
-                    VBoxEventType_OnParallelPortChanged,
-                    VBoxEventType_OnStorageControllerChanged,
-                    VBoxEventType_OnMediumChanged,
-                    VBoxEventType_OnVRDEServerChanged,
-                    VBoxEventType_OnUSBControllerChanged,
-                    VBoxEventType_OnUSBDeviceStateChanged,
-                    VBoxEventType_OnSharedFolderChanged,
-                    VBoxEventType_OnRuntimeError,
-                    VBoxEventType_OnCanShowWindow,
-                    VBoxEventType_OnShowWindow
-                };
+            static const ULONG s_auInterestingEvents[] =
+            {
+                VBoxEventType_OnMousePointerShapeChanged,
+                VBoxEventType_OnMouseCapabilityChanged,
+                VBoxEventType_OnKeyboardLedsChanged,
+                VBoxEventType_OnStateChanged,
+                VBoxEventType_OnAdditionsStateChanged,
+                VBoxEventType_OnNetworkAdapterChanged,
+                VBoxEventType_OnSerialPortChanged,
+                VBoxEventType_OnParallelPortChanged,
+                VBoxEventType_OnStorageControllerChanged,
+                VBoxEventType_OnMediumChanged,
+                VBoxEventType_OnVRDEServerChanged,
+                VBoxEventType_OnUSBControllerChanged,
+                VBoxEventType_OnUSBDeviceStateChanged,
+                VBoxEventType_OnSharedFolderChanged,
+                VBoxEventType_OnRuntimeError,
+                VBoxEventType_OnCanShowWindow,
+                VBoxEventType_OnShowWindow
+            };
             SAFEARRAY *interestingEventsSA = NULL;
             IEventListener *consoleListener = NULL;
 
             /* The VirtualBox API expects enum values as VT_I4, which in the
              * future can be hopefully relaxed. */
-            interestingEventsSA = g_pVBoxFuncs->pfnSafeArrayCreateVector(VT_I4, 0, sizeof(interestingEvents) / sizeof(interestingEvents[0]));
-            g_pVBoxFuncs->pfnSafeArrayCopyInParamHelper(interestingEventsSA, &interestingEvents, sizeof(interestingEvents));
+            interestingEventsSA = g_pVBoxFuncs->pfnSafeArrayCreateVector(VT_I4, 0,
+                                                                           sizeof(s_auInterestingEvents)
+                                                                         / sizeof(s_auInterestingEvents[0]));
+            g_pVBoxFuncs->pfnSafeArrayCopyInParamHelper(interestingEventsSA, &s_auInterestingEvents,
+                                                        sizeof(s_auInterestingEvents));
 
             rc = IEventSource_CreateListener(es, &consoleListener);
             if (SUCCEEDED(rc) && consoleListener)
@@ -641,23 +648,17 @@ static void registerPassiveEventListener(IVirtualBox *virtualBox, ISession *sess
 #endif
                 }
                 else
-                {
                     printf("Failed to register event listener.\n");
-                }
                 IEventSource_UnregisterListener(es, (IEventListener *)consoleListener);
                 IEventListener_Release(consoleListener);
             }
             else
-            {
                 printf("Failed to create an event listener instance.\n");
-            }
             g_pVBoxFuncs->pfnSafeArrayDestroy(interestingEventsSA);
             IEventSource_Release(es);
         }
         else
-        {
             printf("Failed to get the event source instance.\n");
-        }
         IConsole_Release(console);
     }
 }
@@ -673,16 +674,14 @@ static void registerPassiveEventListener(IVirtualBox *virtualBox, ISession *sess
 static void PrintErrorInfo(const char *pszExecutable, const char *pszErrorMsg, HRESULT rc)
 {
     IErrorInfo *ex;
-    HRESULT rc2 = S_OK;
+    HRESULT rc2;
     fprintf(stderr, "%s: %s (rc=%#010x)\n", pszExecutable, pszErrorMsg, (unsigned)rc);
     rc2 = g_pVBoxFuncs->pfnGetException(&ex);
-    if (SUCCEEDED(rc2))
+    if (SUCCEEDED(rc2) && ex)
     {
         IVirtualBoxErrorInfo *ei;
         rc2 = IErrorInfo_QueryInterface(ex, &IID_IVirtualBoxErrorInfo, (void **)&ei);
-        if (FAILED(rc2))
-            ei = NULL;
-        if (ei)
+        if (SUCCEEDED(rc2) && ei != NULL)
         {
             /* got extended error info, maybe multiple infos */
             do
@@ -715,8 +714,7 @@ static void PrintErrorInfo(const char *pszExecutable, const char *pszErrorMsg, H
                     ei_next = NULL;
                 IVirtualBoxErrorInfo_Release(ei);
                 ei = ei_next;
-            }
-            while (ei);
+            } while (ei);
         }
 
         IErrorInfo_Release(ex);
@@ -739,6 +737,7 @@ static void startVM(const char *argv0, IVirtualBox *virtualBox, ISession *sessio
     IProgress *progress   = NULL;
     BSTR env              = NULL;
     BSTR sessionType;
+    SAFEARRAY *groupsSA = g_pVBoxFuncs->pfnSafeArrayOutParamAlloc();
 
     rc = IVirtualBox_FindMachine(virtualBox, id, &machine);
     if (FAILED(rc) || !machine)
@@ -747,6 +746,31 @@ static void startVM(const char *argv0, IVirtualBox *virtualBox, ISession *sessio
         return;
     }
 
+    rc = IMachine_get_Groups(machine, ComSafeArrayAsOutTypeParam(groupsSA, BSTR));
+    if (SUCCEEDED(rc))
+    {
+        BSTR *groups = NULL;
+        ULONG cbGroups = 0;
+        ULONG i, cGroups;
+        g_pVBoxFuncs->pfnSafeArrayCopyOutParamHelper((void **)&groups, &cbGroups, VT_BSTR, groupsSA);
+        g_pVBoxFuncs->pfnSafeArrayDestroy(groupsSA);
+        cGroups = cbGroups / sizeof(groups[0]);
+        for (i = 0; i < cGroups; ++i)
+        {
+            /* Note that the use of %S might be tempting, but it is not
+             * available on all platforms, and even where it is usable it
+             * may depend on correct compiler options to make wchar_t a
+             * 16 bit number. So better play safe and use UTF-8. */
+            char *group;
+            g_pVBoxFuncs->pfnUtf16ToUtf8(groups[i], &group);
+            printf("Groups[%d]: %s\n", i, group);
+            g_pVBoxFuncs->pfnUtf8Free(group);
+        }
+        for (i = 0; i < cGroups; ++i)
+            g_pVBoxFuncs->pfnComUnallocString(groups[i]);
+        g_pVBoxFuncs->pfnArrayOutFree(groups);
+    }
+
     g_pVBoxFuncs->pfnUtf8ToUtf16("gui", &sessionType);
     rc = IMachine_LaunchVMProcess(machine, session, sessionType, env, &progress);
     g_pVBoxFuncs->pfnUtf16Free(sessionType);
@@ -785,10 +809,10 @@ static void startVM(const char *argv0, IVirtualBox *virtualBox, ISession *sessio
             /* Kick off the event listener demo part, which is quite separate.
              * Ignore it if you need a more basic sample. */
 #ifdef USE_ACTIVE_EVENT_LISTENER
-            registerActiveEventListener(virtualBox, session, id);
-#else /* !USE_ACTIVE_EVENT_LISTENER */
-            registerPassiveEventListener(virtualBox, session, id);
-#endif /* !USE_ACTIVE_EVENT_LISTENER */
+            registerActiveEventListener(virtualBox, session);
+#else
+            registerPassiveEventListener(session);
+#endif
         }
         IProgress_Release(progress);
     }
@@ -834,6 +858,7 @@ static void listVMs(const char *argv0, IVirtualBox *virtualBox, ISession *sessio
 
     if (!machineCnt)
     {
+        g_pVBoxFuncs->pfnArrayOutFree(machines);
         printf("\tNo VMs\n");
         return;
     }
@@ -843,7 +868,6 @@ static void listVMs(const char *argv0, IVirtualBox *virtualBox, ISession *sessio
     /*
      * Iterate through the collection.
      */
-
     for (i = 0; i < machineCnt; ++i)
     {
         IMachine *machine      = machines[i];
@@ -871,9 +895,7 @@ static void listVMs(const char *argv0, IVirtualBox *virtualBox, ISession *sessio
             g_pVBoxFuncs->pfnUtf8Free(machineName);
         }
         else
-        {
             printf("\tName:        <inaccessible>\n");
-        }
 
         {
             BSTR uuidUtf16;
@@ -929,9 +951,8 @@ static void listVMs(const char *argv0, IVirtualBox *virtualBox, ISession *sessio
     /*
      * Let the user chose a machine to start.
      */
-
     printf("Type Machine# to start (0 - %u) or 'quit' to do nothing: ",
-        (unsigned)(machineCnt - 1));
+           (unsigned)(machineCnt - 1));
     fflush(stdout);
 
     if (scanf("%u", &start_id) == 1 && start_id < machineCnt)
@@ -951,18 +972,14 @@ static void listVMs(const char *argv0, IVirtualBox *virtualBox, ISession *sessio
     /*
      * Don't forget to release the objects in the array.
      */
-
     for (i = 0; i < machineCnt; ++i)
     {
         IMachine *machine = machines[i];
 
         if (machine)
-        {
             IMachine_Release(machine);
-        }
     }
-    if (machines)
-        free(machines);
+    g_pVBoxFuncs->pfnArrayOutFree(machines);
 }
 
 /* Main - Start the ball rolling. */
@@ -975,7 +992,8 @@ int main(int argc, char **argv)
     ULONG       revision         = 0;
     BSTR        versionUtf16     = NULL;
     BSTR        homefolderUtf16  = NULL;
-    HRESULT    rc;     /* Result code of various function (method) calls. */
+    HRESULT     rc;     /* Result code of various function (method) calls. */
+    (void)argc;
 
     printf("Starting main()\n");
 
@@ -1033,7 +1051,6 @@ int main(int argc, char **argv)
      */
 
     /* 1. Revision */
-
     rc = IVirtualBox_get_Revision(vbox, &revision);
     if (SUCCEEDED(rc))
         printf("\tRevision: %u\n", revision);
@@ -1041,7 +1058,6 @@ int main(int argc, char **argv)
         PrintErrorInfo(argv[0], "GetRevision() failed", rc);
 
     /* 2. Version */
-
     rc = IVirtualBox_get_Version(vbox, &versionUtf16);
     if (SUCCEEDED(rc))
     {
@@ -1055,7 +1071,6 @@ int main(int argc, char **argv)
         PrintErrorInfo(argv[0], "GetVersion() failed", rc);
 
     /* 3. Home Folder */
-
     rc = IVirtualBox_get_HomeFolder(vbox, &homefolderUtf16);
     if (SUCCEEDED(rc))
     {
@@ -1076,7 +1091,6 @@ int main(int argc, char **argv)
     /*
      * Do as mom told us: always clean up after yourself.
      */
-
 #ifdef USE_ACTIVE_EVENT_LISTENER
 # ifdef WIN32
     if (g_pTInfoIEventListener)
