@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -35,10 +35,10 @@
 #include "USBControllerImpl.h"              // required for MachineImpl.h to compile on Windows
 #include "BandwidthControlImpl.h"
 #include "BandwidthGroupImpl.h"
-#include "VBox/settings.h"
 #ifdef VBOX_WITH_RESOURCE_USAGE_API
 #include "Performance.h"
 #include "PerformanceImpl.h"
+#include "ThreadTask.h"
 #endif /* VBOX_WITH_RESOURCE_USAGE_API */
 
 // generated header
@@ -55,6 +55,10 @@
 
 #include "MachineWrap.h"
 
+/** @todo r=klaus after moving the various Machine settings structs to
+ * MachineImpl.cpp it should be possible to eliminate this include. */
+#include <VBox/settings.h>
+
 // defines
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -75,16 +79,6 @@ class HostUSBDevice;
 class StorageController;
 class SessionMachine;
 
-namespace settings
-{
-    class MachineConfigFile;
-    struct Snapshot;
-    struct Hardware;
-    struct Storage;
-    struct StorageController;
-    struct MachineRegistryEntry;
-}
-
 // Machine class
 ////////////////////////////////////////////////////////////////////////////////
 //
@@ -228,8 +222,6 @@ public:
     struct UserData
     {
         settings::MachineUserData s;
-        typedef  std::vector<uint8_t> IconBlob;
-        IconBlob mIcon;
     };
 
     /**
@@ -285,10 +277,13 @@ public:
         BOOL                mPAEEnabled;
         settings::Hardware::LongModeType mLongMode;
         BOOL                mTripleFaultReset;
+        BOOL                mAPIC;
+        BOOL                mX2APIC;
         ULONG               mCPUCount;
         BOOL                mCPUHotPlugEnabled;
         ULONG               mCpuExecutionCap;
         uint32_t            mCpuIdPortabilityLevel;
+        Utf8Str             mCpuProfile;
         BOOL                mAccelerate3DEnabled;
         BOOL                mHPETEnabled;
 
@@ -313,6 +308,7 @@ public:
         PointingHIDType_T   mPointingHIDType;
         ChipsetType_T       mChipsetType;
         ParavirtProvider_T  mParavirtProvider;
+        Utf8Str             mParavirtDebug;
         BOOL                mEmulatedUSBCardReaderEnabled;
 
         BOOL                mIOCacheEnabled;
@@ -498,8 +494,10 @@ public:
      * Intended to be used only after doing addCaller() manually and locking it
      * for reading.
      */
+    Utf8Str i_getOSTypeId() const { return mUserData->s.strOsType; }
     ChipsetType_T i_getChipsetType() const { return mHWData->mChipsetType; }
     ParavirtProvider_T i_getParavirtProvider() const { return mHWData->mParavirtProvider; }
+    Utf8Str i_getParavirtDebug() const { return mHWData->mParavirtDebug; }
 
     void i_setModified(uint32_t fl, bool fAllowStateModification = true);
     void i_setModifiedLock(uint32_t fl, bool fAllowStateModification = true);
@@ -639,7 +637,10 @@ protected:
     HRESULT i_loadSnapshot(const settings::Snapshot &data,
                            const Guid &aCurSnapshotId,
                            Snapshot *aParentSnapshot);
-    HRESULT i_loadHardware(const settings::Hardware &data, const settings::Debugging *pDbg,
+    HRESULT i_loadHardware(const Guid *puuidRegistry,
+                           const Guid *puuidSnapshot,
+                           const settings::Hardware &data,
+                           const settings::Debugging *pDbg,
                            const settings::Autostart *pAutostart);
     HRESULT i_loadDebugging(const settings::Debugging *pDbg);
     HRESULT i_loadAutostart(const settings::Autostart *pAutostart);
@@ -790,53 +791,32 @@ protected:
      * the static Machine::taskHandler, which then calls the handler() method
      * in here (implemented by the subclasses).
      */
-    struct Task
+    class Task : public ThreadTask
     {
+    public:
         Task(Machine *m, Progress *p, const Utf8Str &t)
-            : m_pMachine(m),
+            : ThreadTask(t),
+              m_pMachine(m),
               m_machineCaller(m),
               m_pProgress(p),
-              m_strTaskName(t),
               m_machineStateBackup(m->mData->mMachineState) // save the current machine state
         {}
-
-        HRESULT createThread()
-        {
-            int vrc = RTThreadCreate(NULL,
-                                     taskHandler,
-                                     (void *)this,
-                                     0,
-                                     RTTHREADTYPE_MAIN_WORKER,
-                                     0,
-                                     m_strTaskName.c_str());
-            if (RT_FAILURE(vrc))
-            {
-                HRESULT rc = Machine::i_setErrorStatic(E_FAIL, Machine::tr("Could not create thread \"%s\" (%Rrc)"), m_strTaskName.c_str(), vrc);
-                delete this;
-                return rc;
-            }
-            return S_OK;
-        }
+        virtual ~Task(){}
 
         void modifyBackedUpState(MachineState_T s)
         {
             *const_cast<MachineState_T *>(&m_machineStateBackup) = s;
         }
 
-        virtual void handler() = 0;
-
-        ComObjPtr<Machine>       m_pMachine;
+        ComObjPtr<Machine>              m_pMachine;
         AutoCaller                      m_machineCaller;
         ComObjPtr<Progress>             m_pProgress;
-        Utf8Str                         m_strTaskName;
         const MachineState_T            m_machineStateBackup;
     };
 
-    struct DeleteConfigTask;
+    class DeleteConfigTask;
     void i_deleteConfigHandler(DeleteConfigTask &task);
 
-    static DECLCALLBACK(int) taskHandler(RTTHREAD thread, void *pvUser);
-
     friend class SessionMachine;
     friend class SnapshotMachine;
     friend class Appliance;
@@ -872,6 +852,8 @@ private:
     HRESULT setCPUExecutionCap(ULONG aCPUExecutionCap);
     HRESULT getCPUIDPortabilityLevel(ULONG *aCPUIDPortabilityLevel);
     HRESULT setCPUIDPortabilityLevel(ULONG aCPUIDPortabilityLevel);
+    HRESULT getCPUProfile(com::Utf8Str &aCPUProfile);
+    HRESULT setCPUProfile(const com::Utf8Str &aCPUProfile);
     HRESULT getMemorySize(ULONG *aMemorySize);
     HRESULT setMemorySize(ULONG aMemorySize);
     HRESULT getMemoryBalloonSize(ULONG *aMemoryBalloonSize);
@@ -930,6 +912,7 @@ private:
     HRESULT getAudioAdapter(ComPtr<IAudioAdapter> &aAudioAdapter);
     HRESULT getStorageControllers(std::vector<ComPtr<IStorageController> > &aStorageControllers);
     HRESULT getSettingsFilePath(com::Utf8Str &aSettingsFilePath);
+    HRESULT getSettingsAuxFilePath(com::Utf8Str &aSettingsAuxFilePath);
     HRESULT getSettingsModified(BOOL *aSettingsModified);
     HRESULT getSessionState(SessionState_T *aSessionState);
     HRESULT getSessionType(SessionType_T *aSessionType);
@@ -957,6 +940,8 @@ private:
     HRESULT setTeleporterPassword(const com::Utf8Str &aTeleporterPassword);
     HRESULT getParavirtProvider(ParavirtProvider_T *aParavirtProvider);
     HRESULT setParavirtProvider(ParavirtProvider_T aParavirtProvider);
+    HRESULT getParavirtDebug(com::Utf8Str &aParavirtDebug);
+    HRESULT setParavirtDebug(const com::Utf8Str &aParavirtDebug);
     HRESULT getFaultToleranceState(FaultToleranceState_T *aFaultToleranceState);
     HRESULT setFaultToleranceState(FaultToleranceState_T aFaultToleranceState);
     HRESULT getFaultTolerancePort(ULONG *aFaultTolerancePort);
@@ -1286,8 +1271,11 @@ public:
     DECLARE_PROTECT_FINAL_CONSTRUCT()
 
     BEGIN_COM_MAP(SessionMachine)
-        VBOX_DEFAULT_INTERFACE_ENTRIES(IMachine)
+        COM_INTERFACE_ENTRY(ISupportErrorInfo)
+        COM_INTERFACE_ENTRY(IMachine)
+        COM_INTERFACE_ENTRY2(IDispatch, IMachine)
         COM_INTERFACE_ENTRY(IInternalMachineControl)
+        VBOX_TWEAK_INTERFACE_ENTRY(IMachine)
     END_COM_MAP()
 
     DECLARE_EMPTY_CTOR_DTOR(SessionMachine)
@@ -1428,15 +1416,11 @@ private:
         void *mDeleteSnapshotInfo;
     };
 
-    struct SaveStateTask;
-    struct SnapshotTask;
-    struct TakeSnapshotTask;
-    struct DeleteSnapshotTask;
-    struct RestoreSnapshotTask;
-
-    friend struct TakeSnapshotTask;
-    friend struct DeleteSnapshotTask;
-    friend struct RestoreSnapshotTask;
+    class SaveStateTask;
+    class SnapshotTask;
+    class TakeSnapshotTask;
+    class DeleteSnapshotTask;
+    class RestoreSnapshotTask;
 
     void i_saveStateHandler(SaveStateTask &aTask);
 
@@ -1540,7 +1524,10 @@ public:
     DECLARE_PROTECT_FINAL_CONSTRUCT()
 
     BEGIN_COM_MAP(SnapshotMachine)
-        VBOX_DEFAULT_INTERFACE_ENTRIES(IMachine)
+        COM_INTERFACE_ENTRY(ISupportErrorInfo)
+        COM_INTERFACE_ENTRY(IMachine)
+        COM_INTERFACE_ENTRY2(IDispatch, IMachine)
+        VBOX_TWEAK_INTERFACE_ENTRY(IMachine)
     END_COM_MAP()
 
     DECLARE_EMPTY_CTOR_DTOR(SnapshotMachine)
@@ -1556,7 +1543,6 @@ public:
                              const settings::Hardware &hardware,
                              const settings::Debugging *pDbg,
                              const settings::Autostart *pAutostart,
-                             const settings::Storage &storage,
                              IN_GUID aSnapshotId,
                              const Utf8Str &aStateFilePath);
     void uninit();
