@@ -1,10 +1,11 @@
+/* $Id: VBoxDriversRegister.cpp $ */
 /** @file
  *
  * Main driver registration.
  */
 
 /*
- * Copyright (C) 2006-2013 Oracle Corporation
+ * Copyright (C) 2006-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -16,18 +17,18 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include "MouseImpl.h"
 #include "KeyboardImpl.h"
 #include "DisplayImpl.h"
 #include "VMMDev.h"
-#include "AudioSnifferInterface.h"
-#include "Nvram.h"
-#ifdef VBOX_WITH_USB_VIDEO
-# include "UsbWebcamInterface.h"
+#ifdef VBOX_WITH_VRDE_AUDIO
+# include "DrvAudioVRDE.h"
 #endif
+#include "Nvram.h"
+#include "UsbWebcamInterface.h"
 #ifdef VBOX_WITH_USB_CARDREADER
 # include "UsbCardReader.h"
 #endif
@@ -68,20 +69,18 @@ extern "C" DECLEXPORT(int) VBoxDriversRegister(PCPDMDRVREGCB pCallbacks, uint32_
     rc = pCallbacks->pfnRegister(pCallbacks, &VMMDev::DrvReg);
     if (RT_FAILURE(rc))
         return rc;
-
-    rc = pCallbacks->pfnRegister(pCallbacks, &AudioSniffer::DrvReg);
+#ifdef VBOX_WITH_VRDE_AUDIO
+    rc = pCallbacks->pfnRegister(pCallbacks, &AudioVRDE::DrvReg);
     if (RT_FAILURE(rc))
         return rc;
-
+#endif
     rc = pCallbacks->pfnRegister(pCallbacks, &Nvram::DrvReg);
     if (RT_FAILURE(rc))
         return rc;
 
-#ifdef VBOX_WITH_USB_VIDEO
     rc = pCallbacks->pfnRegister(pCallbacks, &EmWebcam::DrvReg);
     if (RT_FAILURE(rc))
         return rc;
-#endif
 
 #ifdef VBOX_WITH_USB_CARDREADER
     rc = pCallbacks->pfnRegister(pCallbacks, &UsbCardReader::DrvReg);
