@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Oracle Corporation
+ * Copyright (C) 2006-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -24,9 +24,10 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include <iprt/types.h>
 #include <iprt/stdarg.h>
 #include <iprt/ctype.h>
@@ -34,17 +35,22 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#ifdef _MSC_VER
+# include <direct.h>
+#else
+# include <unistd.h>
+#endif
 
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
-/** When set, no output is produced.  Very useful when debugging ths code. */
-static bool g_fQuiet = false;
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 /** The file we're currently parsing. */
 static const char *g_pszCurFile;
 /** The current line number. */
 static unsigned g_iLine;
+/** The current output file. */
+static FILE *g_pCurOutFile;
 
 
 /**
@@ -53,7 +59,7 @@ static unsigned g_iLine;
  * @param   pszFormat           The message.
  * @param   ...                 Format arguments.
  */
-static void ParseError(const char *pszFormat, ...)
+static DECL_NO_RETURN(void) ParseError(const char *pszFormat, ...)
 {
     va_list va;
     va_start(va, pszFormat);
@@ -191,7 +197,7 @@ static char *SplitDecompField(char **ppszType)
     if (!psz)
     {
         ParseError("Bad Decomposition Type/Mappings\n");
-        return *ppszType;
+        /* not reached: return *ppszType; */
     }
     *psz++ = '\0';
 
@@ -249,13 +255,13 @@ static RTUNICP ToRange(const char *psz, PRTUNICP pLast)
             if (pszEnd && *pszEnd)
             {
                 ParseError("failed converting '%s' to a number!\n", psz);
-                return ~(RTUNICP)0;
+                /* not reached: return ~(RTUNICP)0;*/
             }
         }
         else
         {
             ParseError("failed converting '%s' to a number!\n", psz);
-            return ~(RTUNICP)0;
+            /* not reached: return ~(RTUNICP)0; */
         }
     }
     *pLast = (RTUNICP)ulLast;
@@ -292,7 +298,7 @@ static PRTUNICP ToMapping(char *psz, unsigned *pcEntries, unsigned cMax)
         if (i >= cMax)
         {
             ParseError("Too many mappings.\n");
-            break;
+            /* not reached: break; */
         }
         if (i >= cAlloc)
         {
@@ -586,7 +592,7 @@ static int ReadUnicodeData(const char *pszBasePath, const char *pszFilename)
         if (CodePoint >= RT_ELEMENTS(g_aCPInfo))
         {
             ParseError("U+05X is out of range\n", CodePoint);
-            continue;
+            /* not reached: continue;*/
         }
 
         /* catchup? */
@@ -595,8 +601,8 @@ static int ReadUnicodeData(const char *pszBasePath, const char *pszFilename)
         if (i != CodePoint)
         {
             ParseError("i=%d CodePoint=%u\n", i, CodePoint);
-            CloseFile(pFile);
-            return 1;
+            /* not reached: CloseFile(pFile);
+            return 1; */
         }
 
         /* this one */
@@ -668,19 +674,19 @@ static int GenerateExcludedData(void)
 static int YesNoMaybePropertyValue(char **ppszNextField)
 {
     if (!**ppszNextField)
-    {
         ParseError("Missing Y/N/M field\n");
-        return 0;
+    else
+    {
+        char *psz = NextField(ppszNextField);
+        if (!strcmp(psz, "N"))
+            return 0;
+        if (!strcmp(psz, "Y"))
+            return 1;
+        if (!strcmp(psz, "M"))
+            return 2;
+        ParseError("Unexpected Y/N/M value: '%s'\n",  psz);
     }
-    char *psz = NextField(ppszNextField);
-    if (!strcmp(psz, "N"))
-        return 0;
-    if (!strcmp(psz, "Y"))
-        return 1;
-    if (!strcmp(psz, "M"))
-        return 2;
-    ParseError("Unexpected Y/N/M value: '%s'\n",  psz);
-    return 0;
+    /* not reached: return 0; */
 }
 
 
@@ -705,15 +711,16 @@ static int YesNoMaybePropertyValueInv(char **ppszNextField)
 /**
  * Applies a property to a code point.
  *
- * @param   StartCP     The code point.
- * @param   pszProperty The property name.
+ * @param   StartCP         The code point.
+ * @param   pszProperty     The property name.
+ * @param   pszNextField    The next field.
  */
 static void ApplyProperty(RTUNICP StartCP, const char *pszProperty, char *pszNextField)
 {
     if (StartCP >= RT_ELEMENTS(g_aCPInfo))
     {
         ParseError("U+%06X is out of the g_aCPInfo range.\n", StartCP);
-        return;
+        /* not reached: return; */
     }
     struct CPINFO *pCPInfo = &g_aCPInfo[StartCP];
     /* string switch */
@@ -784,7 +791,7 @@ static void ApplyProperty(RTUNICP StartCP, const char *pszProperty, char *pszNex
     else
     {
         ParseError("Unknown property '%s'\n", pszProperty);
-        return;
+        /* not reached: return; */
     }
 
     if (pszNextField && *pszNextField)
@@ -826,7 +833,7 @@ static int ReadProperties(const char *pszBasePath, const char *pszFilename)
         if (!*pszProperty)
         {
             ParseError("no property field.\n");
-            continue;
+            /* not reached: continue; */
         }
 
         RTUNICP LastCP;
@@ -883,6 +890,7 @@ static bool CalcFlags(struct CPINFO *pInfo, char *pszFlags)
         AppendFlag(pszFlags, "RTUNI_LOWER");
     //if (pInfo->???)
     //    AppendFlag(pszFlags, "RTUNI_BSPACE");
+#if 0
     if (pInfo->fInvNFD_QC != 0 || pInfo->fInvNFC_QC != 0)
     {
         AppendFlag(pszFlags, "RTUNI_QC_NFX");
@@ -893,6 +901,7 @@ static bool CalcFlags(struct CPINFO *pInfo, char *pszFlags)
     }
     else if (pInfo->paDecompositionMapping && !*pInfo->pszDecompositionType)
         fprintf(stderr, "uniread: U+%05X is not QC_NFX but has canonical mappings.\n", pInfo->CodePoint);
+#endif
 
     if (!*pszFlags)
     {
@@ -904,6 +913,43 @@ static bool CalcFlags(struct CPINFO *pInfo, char *pszFlags)
 }
 
 
+/**
+ * Closes the primary output stream.
+ */
+static int Stream1Close(void)
+{
+    if (g_pCurOutFile && g_pCurOutFile != stdout && g_pCurOutFile != stderr)
+    {
+        if (fclose(g_pCurOutFile) != 0)
+        {
+            fprintf(stderr, "Error closing output file.\n");
+            return -1;
+        }
+    }
+    g_pCurOutFile = NULL;
+    return 0;
+}
+
+
+/**
+ * Initializes the 1st stream to output to a given file.
+ */
+static int Stream1Init(const char *pszName)
+{
+    int rc = Stream1Close();
+    if (!rc)
+    {
+        g_pCurOutFile = fopen(pszName, "w");
+        if (!g_pCurOutFile)
+        {
+            fprintf(stderr, "Error opening output file '%s'.\n", pszName);
+            rc = -1;
+        }
+    }
+    return rc;
+}
+
+
 /**
  * printf wrapper for the primary output stream.
  *
@@ -916,10 +962,7 @@ static int Stream1Printf(const char *pszFormat, ...)
     int     cch;
     va_list va;
     va_start(va, pszFormat);
-    if (!g_fQuiet)
-        cch = vfprintf(stdout, pszFormat, va);
-    else
-        cch = strlen(pszFormat);
+    cch = vfprintf(g_pCurOutFile, pszFormat, va);
     va_end(va);
     return cch;
 }
@@ -973,17 +1016,31 @@ static int Stream2Printf(const char *pszFormat, ...)
 /**
  * Print the unidata.cpp file header and include list.
  */
-int PrintHeader(const char *argv0)
+int PrintHeader(const char *argv0, const char *pszBaseDir)
 {
-    Stream1Printf("/** @file\n"
-                  " *\n"
+    char szBuf[1024];
+    if (!pszBaseDir)
+    {
+        memset(szBuf, 0, sizeof(szBuf));
+#ifdef _MSC_VER
+        if (!_getcwd(szBuf, sizeof(szBuf)))
+#else
+        if (!getcwd(szBuf, sizeof(szBuf)))
+#endif
+            return RTEXITCODE_FAILURE;
+        pszBaseDir = szBuf;
+    }
+
+    Stream1Printf("/* $" "Id" "$ */\n"
+                  "/** @file\n"
                   " * IPRT - Unicode Tables.\n"
                   " *\n"
-                  " * Automatically Generated by %s (" __DATE__ " " __TIME__ ")\n"
+                  " * Automatically Generated from %s\n"
+                  " * by %s (" __DATE__ " " __TIME__ ")\n"
                   " */\n"
                   "\n"
                   "/*\n"
-                  " * Copyright (C) 2006-2010 Oracle Corporation\n"
+                  " * Copyright (C) 2006-2016 Oracle Corporation \n"
                   " *\n"
                   " * This file is part of VirtualBox Open Source Edition (OSE), as\n"
                   " * available from http://www.virtualbox.org. This file is free software;\n"
@@ -1005,7 +1062,7 @@ int PrintHeader(const char *argv0)
                   "\n"
                   "#include <iprt/uni.h>\n"
                   "\n",
-                  argv0);
+                  pszBaseDir, argv0);
     return 0;
 }
 
@@ -1019,7 +1076,7 @@ int PrintFlags(void)
      * Print flags table.
      */
     Stream2Init();
-    Stream2Printf("const RTUNIFLAGSRANGE g_aRTUniFlagsRanges[] =\n"
+    Stream2Printf("RT_DECL_DATA_CONST(const RTUNIFLAGSRANGE) g_aRTUniFlagsRanges[] =\n"
                   "{\n");
     RTUNICP i = 0;
     int iStart = -1;
@@ -1046,7 +1103,7 @@ int PrintFlags(void)
         {
             if (iStart < 0)
             {
-                Stream1Printf("static const uint8_t g_afRTUniFlags0x%06x[] = \n"
+                Stream1Printf("static const uint8_t g_afRTUniFlags0x%06x[] =\n"
                               "{\n", i);
                 iStart = i;
             }
@@ -1068,7 +1125,7 @@ int PrintFlags(void)
 static int PrintUpper(void)
 {
     Stream2Init();
-    Stream2Printf("const RTUNICASERANGE g_aRTUniUpperRanges[] =\n"
+    Stream2Printf("RT_DECL_DATA_CONST(const RTUNICASERANGE) g_aRTUniUpperRanges[] =\n"
                   "{\n");
     RTUNICP i = 0;
     int iStart = -1;
@@ -1094,7 +1151,7 @@ static int PrintUpper(void)
         {
             if (iStart < 0)
             {
-                Stream1Printf("static const RTUNICP g_afRTUniUpper0x%06x[] = \n"
+                Stream1Printf("static const RTUNICP g_afRTUniUpper0x%06x[] =\n"
                               "{\n", i);
                 iStart = i;
             }
@@ -1115,7 +1172,7 @@ static int PrintUpper(void)
 static int PrintLower(void)
 {
     Stream2Init();
-    Stream2Printf("const RTUNICASERANGE g_aRTUniLowerRanges[] =\n"
+    Stream2Printf("RT_DECL_DATA_CONST(const RTUNICASERANGE) g_aRTUniLowerRanges[] =\n"
                   "{\n");
     RTUNICP i = 0;
     int iStart = -1;
@@ -1141,7 +1198,7 @@ static int PrintLower(void)
         {
             if (iStart < 0)
             {
-                Stream1Printf("static const RTUNICP g_afRTUniLower0x%06x[] = \n"
+                Stream1Printf("static const RTUNICP g_afRTUniLower0x%06x[] =\n"
                               "{\n", i);
                 iStart = i;
             }
@@ -1201,9 +1258,6 @@ int main(int argc, char **argv)
             argi++;
             pszBaseDir = argv[argi];
         }
-        else if (   !strcmp(argv[argi], "-q")
-                 || !strcmp(argv[argi], "--quiet"))
-            g_fQuiet = true;
         else
         {
             fprintf(stderr, "uniread: syntax error at '%s': Unknown argument\n", argv[argi]);
@@ -1231,24 +1285,29 @@ int main(int argc, char **argv)
         return rc;
 
     /*
-     * Print stuff.
+     * Produce output files.
      */
-    rc = PrintHeader(argv[0]);
-    if (rc)
-        return rc;
-    rc = PrintFlags();
-    if (rc)
-        return rc;
-    rc = PrintUpper();
-    if (rc)
-        return rc;
-    rc = PrintLower();
-    if (rc)
-        return rc;
+    rc = Stream1Init("unidata-flags.cpp");
+    if (!rc)
+        rc = PrintHeader(argv[0], pszBaseDir);
+    if (!rc)
+        rc = PrintFlags();
+
+    rc = Stream1Init("unidata-upper.cpp");
+    if (!rc)
+        rc = PrintHeader(argv[0], pszBaseDir);
+    if (!rc)
+        rc = PrintUpper();
+
+    rc = Stream1Init("unidata-lower.cpp");
+    if (!rc)
+        rc = PrintHeader(argv[0], pszBaseDir);
+    if (!rc)
+        rc = PrintLower();
+    if (!rc)
+        rc = Stream1Close();
 
     /* done */
-    fflush(stdout);
-
     return rc;
 }
 
