@@ -1055,8 +1055,8 @@ DECLINLINE(int) trpmGCTrap0dHandlerRdTsc(PVM pVM, PVMCPU pVCpu, PCPUMCTXCORE pRe
     }
 
     uint64_t uTicks = TMCpuTickGet(pVCpu);
-    pRegFrame->eax = uTicks;
-    pRegFrame->edx = uTicks >> 32;
+    pRegFrame->eax = RT_LO_U32(uTicks);
+    pRegFrame->edx = RT_HI_U32(uTicks);
     pRegFrame->eip += 2;
     TRPM_EXIT_DBG_HOOK(0xd);
     return trpmGCExitTrap(pVM, pVCpu, VINF_SUCCESS, pRegFrame);
